/*---------------------------------------------------------------------------*\
 * Copyright (C) 2000 AVM GmbH. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
 * ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
 * with the use or performance of this software. See the
 * GNU General Public License for further details.
\*---------------------------------------------------------------------------*/

#ifndef _CAPIBASE_H_
#define _CAPIBASE_H_

#include <semaphore.h>
#include <pthread.h>
#include "aTypes.h"
#include "capi2def.h"

/*===========================================================================*\
    CCAPI20_Base
\*===========================================================================*/

class CCAPI20_Base {
public:
    CCAPI20_Base (void);
    virtual ~CCAPI20_Base (void);

    c_word GetApplID (void);

    void IsInstalled (void);
    void Register (tUInt MsgBufSize, tUInt MaxB3Connection, tUInt MaxB3Blks, tUInt MaxSizeB3);
    void Release  (void);
    void PutMessage (c_message Message);
    void GetMessage (void);

    void GetManufacturer (tUInt Controller); // Not implemented yet!
    void GetVersion      (tUInt Controller); // Not implemented yet!
    void GetSerialNumber (tUInt Controller); // Not implemented yet!
    void GetProfile      (tUInt Controller); // Only allowed after Register

    virtual void IsInstalledComplete (c_info /*info*/) {}
    virtual void RegisterComplete    (c_info ErrorCode) = 0;
    virtual void ReleaseComplete     (c_info ErrorCode) = 0;
    virtual void PutMessageComplete  (c_info ErrorCode) = 0;
    virtual void GetMessageComplete  (c_message Message, c_info ErrorCode) = 0;

    virtual void GetManufacturerComplete (c_message /*lpBuffer*/, c_info /*ErrorCode*/) {}
    virtual void GetVersionComplete      (c_message /*version*/, c_info /*ErrorCode*/) {}
    virtual void GetSerialNumberComplete (c_message /*lpBuffer*/, c_info /*ErrorCode*/) {}
    virtual void GetProfileComplete      (cp_getprofile * /*pProfile*/, c_info /*ErrorCode*/) {}

    void GetMessageThread (void);

private:
    c_word    applID;
    sem_t     semaphore;
    pthread_t thread;
    tUInt     threadStop;
};

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline c_word CCAPI20_Base::GetApplID (void) {
    return applID;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#endif

