/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "CapiChan.h"
#include "ConvCapi.h"
#include "osImport.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define BCHANNELTIMER_WAITTIME              90 // sec

static void BChannelTimer_CallBack (void *pCapiChannel, void *) {
    dhead ("BChannelTimer_CallBack", DEBUG_CCAPI20_Channel);
    dassert (pCapiChannel != 0);
    CCAPI20_Channel *pChan = (CCAPI20_Channel *)pCapiChannel;
    switch (pChan->GetState()) {
    case CCAPI20_Channel::cs_D_Connected:
    case CCAPI20_Channel::cs_B_ConnectPending:
        pChan->DisconnectReq();
        break;
    default:
        // in this states the timer should have been canceled before
        dwarning (0);
        break;
    }
}


/*===========================================================================*\
    CCAPI20_Channel Konstruktor
\*===========================================================================*/

CCAPI20_Channel::CCAPI20_Channel (void) : BChannelTimer (BChannelTimer_CallBack) {
    dhead ("CCAPI20_Channel-Constructor", DEBUG_CCAPI20_Channel);
    SetState (cs_Released);
    SetController (IllegalController);
    SetPLCI (IllegalPLCI);
    SetNCCI (IllegalNCCI);
    NCPIout   = 0;
    Initiator = vFalse;
}


/*===========================================================================*\
    CCAPI20_Channel Destruktor
\*===========================================================================*/

CCAPI20_Channel::~CCAPI20_Channel (void) {
    dhead ("CCAPI20_Channel-Destructor", DEBUG_CCAPI20_Channel);
    dassert (GetState() == cs_Released);
    dassert (GetPLCI() == IllegalPLCI);
    dassert (GetNCCI() == IllegalNCCI);
    dwarning (Initiator == vFalse);
    SetNCPI (0);
    BChannelTimer.CancelTimer();
}


/*===========================================================================*\
    CCAPI20_Channel::SetNCPI
\*===========================================================================*/

void CCAPI20_Channel::SetNCPI (c_struct NCPI) {
    dhead ("CCAPI20_Channel::SetNCPI", DEBUG_CCAPI20_Channel);
    Conv_DuplicateStruct (&NCPIout, NCPI);
}


/*===========================================================================*\
    CCAPI20_Channel::AcceptIncomingCall
\*===========================================================================*/

tBool CCAPI20_Channel::AcceptIncomingCall (void) {
    dhead ("CCAPI20_Channel::AcceptIncomingCall", DEBUG_CCAPI20_Channel);
    // TODO: test if AcceptIncommingCall is not called twice and only in the real right state (e.g. before AlertReq)
    dwarning (GetState() == cs_D_ConnectIndication);
    if (GetState() == cs_D_ConnectIndication) {
        dassert (GetPLCI() != IllegalPLCI);
        if (SetNewPLCIEntry (GetPLCI()) == vTrue) {
            RETURN ('x', vTrue);
        }
        c_dword curPLCI = GetPLCI();
        SetController (IllegalController);
        SetPLCI (IllegalPLCI);
        SetState (cs_Registered);
        CONNECT_RESP (GetNextMsgNr(), curPLCI, &cpV_ConnectResp_Ignore);
    }
    RETURN ('x', vFalse);
}


/*===========================================================================*\
    CCAPI20_Channel::Register
\*===========================================================================*/

void CCAPI20_Channel::Register (void) {
    dhead ("CCAPI20_Channel::Register", DEBUG_CCAPI20_Channel);
    dparams ("t=%x", this);
    dassert (GetState() == cs_Released);
    SetState (cs_Registering);
    REGISTER (1, CAPI_MAX_OUTSTANDING_FRAMES, CAPI_MAX_DATA_SIZE);
}


/*===========================================================================*\
    CCAPI20_Channel::Release
\*===========================================================================*/

void CCAPI20_Channel::Release (void) {
    dhead ("CCAPI20_Channel::Release", DEBUG_CCAPI20_Channel);
    dassert (GetState() > cs_Releasing);
    FreePLCIEntry (GetPLCI());
    SetController (IllegalController);
    SetPLCI (IllegalPLCI);
    SetNCCI (IllegalNCCI);
    SetState (cs_Releasing); // TODO: abfangen, wenn schon released ist?
    Initiator = vFalse;
    RELEASE();
}


/*===========================================================================*\
    CCAPI20_Channel::ConnectReq
\*===========================================================================*/

void CCAPI20_Channel::ConnectReq (c_dword Controller, cp_connectreq *Params) {
    dhead ("CCAPI20_Channel::ConnectReq", DEBUG_CCAPI20_Channel);
    dassert (Params != 0);
    if (GetState() == cs_Registered) {
        dassert (GetController() == IllegalController);
        SetState (cs_D_ConnectRequest);
        SetController (Controller);
        Initiator = vTrue;

        CONNECT_REQ (GetNextMsgNr(), Controller, Params);
    } else {
        DisconnectInd (ci_NoError);
    }
}


/*===========================================================================*\
    CCAPI20_Channel::DataB3Req
\*===========================================================================*/

void CCAPI20_Channel::DataB3Req (cp_datab3 *Params) {
    dhead ("CCAPI20_Channel::DataB3Req", DEBUG_CCAPI20_Channel);
    dassert (Params != 0);
    dwarning (GetState() == cs_Connected);
    if (GetState() == cs_Connected) {
        dassert (GetNCCI() != IllegalNCCI);
        DATA_B3_REQ (GetNextMsgNr(), GetNCCI(), Params);
    }
}


/*===========================================================================*\
    CCAPI20_Channel::DisconnectReq
\*===========================================================================*/

void CCAPI20_Channel::DisconnectReq (void) {
    dhead ("CCAPI20_Channel::DisconnectReq", DEBUG_CCAPI20_Channel);
    dwarning (GetState() >= cs_Registered);
    switch (GetState()) {
    case cs_D_ConnectRequest:
        SetState (cs_D_DisconnectPending);
        break;

    case cs_B_DisconnectPending:
    case cs_D_DisconnectPending:
        /*----- Do Nothing!! -----*/
        break;

    case cs_Connected:
        dassert (GetNCCI() != IllegalNCCI);
        SetState (cs_B_DisconnectPending);
        DISCONNECT_B3_REQ (GetNextMsgNr(), GetNCCI(), NCPIout);
        break;

    case cs_Released:
    case cs_Releasing:
    case cs_Registering:
        dwarning (0); // DisconnectReq should not been called in this states!
        // no break!
    case cs_Registered:
        DisconnectInd (ci_NoError);
        break;

    default:
        dassert (GetPLCI() != IllegalPLCI);
        dwarning (GetNCCI() == IllegalNCCI);
        SetState (cs_D_DisconnectPending);
        DISCONNECT_REQ (GetNextMsgNr(), GetPLCI());
        break;
    }
}


/*===========================================================================*\
    CCAPI20_Channel::ConnectResp
\*===========================================================================*/

void CCAPI20_Channel::ConnectResp (cp_connectresp *Params) {
    dhead ("CCAPI20_Channel::ConnectResp", DEBUG_CCAPI20_Channel);
    dassert (Params != 0);
    dwarning (GetState() == cs_D_ConnectIndication);
    dassert (Initiator == vFalse);

    if (GetState() == cs_D_ConnectIndication) {
        dassert (GetPLCI() != IllegalPLCI);
        if (Params->Reject == creject_Accept) {
            SetState (cs_D_ConnectPending);
            CONNECT_RESP (GetNextMsgNr(), GetPLCI(), Params);
        } else {
            c_dword curPLCI = GetPLCI();
            SetController (IllegalController);
            SetPLCI (IllegalPLCI);
            SetState (cs_Registered);
            CONNECT_RESP (GetNextMsgNr(), curPLCI, &cpV_ConnectResp_Ignore);
        }
    }
}


/*===========================================================================*\
    CCAPI20_Channel::DataB3Resp
\*===========================================================================*/

void CCAPI20_Channel::DataB3Resp (c_word DataHandle) {
    dhead ("CCAPI20_Channel::DataB3Resp", DEBUG_CCAPI20_Channel);
    dassert (GetState() == cs_Connected);
    dassert (GetNCCI() != IllegalNCCI);
    DATA_B3_RESP (GetNextMsgNr(), GetNCCI(), DataHandle);
}


/*===========================================================================*\
    CCAPI20_Channel::REGISTER_COMPLETE
\*===========================================================================*/

void CCAPI20_Channel::REGISTER_COMPLETE (c_info ErrorCode) {
    dhead ("CCAPI20_Channel::REGISTER_COMPLETE", DEBUG_CCAPI20_Channel);
    dwarning (ErrorCode == ci_NoError);
    dassert (GetState() == cs_Registering);
    SetState (CapiInfoIsError (ErrorCode) ? cs_Released : cs_Registered);
    Register_Complete (ErrorCode);
}


/*===========================================================================*\
    CCAPI20_Channel::RELEASE_COMPLETE
\*===========================================================================*/

void CCAPI20_Channel::RELEASE_COMPLETE (c_info ErrorCode) {
    dhead ("CCAPI20_Channel::RELEASE_COMPLETE", DEBUG_CCAPI20_Channel);
    dwarning (GetState() == cs_Releasing);
    FreePLCIEntry (GetPLCI());
    SetState (cs_Released);
    SetController (IllegalController);
    SetPLCI (IllegalPLCI);
    SetNCCI (IllegalNCCI);
    Initiator  = vFalse;
    Release_Complete (ErrorCode);
}


/*===========================================================================*\
    CCAPI20_Channel - Conf-Messages
\*===========================================================================*/

void CCAPI20_Channel::CONNECT_CONF (c_word, c_dword PLCI, c_info Info) {
    dhead ("CCAPI20_Channel::CONNECT_CONF", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (GetState() != cs_D_ConnectRequest) {
        dwarning (0);   // Im falschen Status!
        if (CapiInfoIsSuccess (Info)) {
            DISCONNECT_REQ (GetNextMsgNr(), PLCI);
        }
        return;
    }
    dassert (GetController() != IllegalController);
    dassert (GetPLCI() == IllegalPLCI);
    dassert (GetNCCI() == IllegalNCCI);
    dassert (Initiator == vTrue);
    if (CapiInfoIsSuccess (Info)) {
        SetPLCI (PLCI);
        SetState (cs_D_ConnectPending);
    } else {
        SetController (IllegalController);
        SetState (cs_Registered);
        Initiator = vFalse;
        DisconnectInd (Info);
    }
}

void CCAPI20_Channel::CONNECT_B3_CONF (c_word, c_dword NCCI, c_info Info) {
    dhead ("CCAPI20_Channel::CONNECT_B3_CONF", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if ((GetState() == cs_D_Connected) && ((GetPLCIFromNCCI (NCCI) == GetPLCI()))) {
        dassert (GetNCCI() == IllegalNCCI);
        dassert (Initiator == vFalse);
        if (CapiInfoIsSuccess (Info)) {
            SetNCCI (NCCI);
            SetState (cs_B_ConnectPending);
            dprint ("curstate=%x, ncci=%x, NCCI=%x ", GetState(), GetNCCI(), NCCI);
            return;
        }
    } else {
        dwarning (0);
    }
    DISCONNECT_REQ (GetNextMsgNr(), GetPLCIFromNCCI (NCCI));
}

void CCAPI20_Channel::DATA_B3_CONF (c_word, c_dword NCCI, cp_datab3conf *Params) {
    dhead ("CCAPI20_Channel::DATA_B3_CONF", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    dassert (Params != 0);
    if (NCCI != GetNCCI()) {
        dwarning (0);
        DISCONNECT_B3_REQ (GetNextMsgNr(), NCCI, 0);
        return;
    }
    dassert (GetState() == cs_Connected);
    DataB3Conf (Params);
}

void CCAPI20_Channel::DISCONNECT_B3_CONF (c_word, c_dword NCCI, c_info Info) {
    dhead ("CCAPI20_Channel::DISCONNECT_B3_CONF", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (NCCI == GetNCCI()) {
        dwarning (GetState() > cs_D_Connected);
        if (CapiInfoIsSuccess (Info)) {
            SetState (cs_B_DisconnectPending);
            return;
        }
    } else {
        dwarning (0);
    }
    DISCONNECT_REQ (GetNextMsgNr(), GetPLCIFromNCCI (NCCI));
}

void CCAPI20_Channel::DISCONNECT_CONF (c_word, c_dword PLCI, c_info Info) {
    dhead ("CCAPI20_Channel::DISCONNECT_CONF", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (PLCI == GetPLCI()) {
        if (CapiInfoIsSuccess (Info)) {
            SetState (cs_D_DisconnectPending);
            BChannelTimer.CancelTimer();
        } else {
            dassert (0);
            //TODO: Was hier bei einem Fehler machen?
        }
    }
}

void CCAPI20_Channel::LISTEN_CONF (c_word, c_dword Controller, c_info Info) {
    dhead ("CCAPI20_Channel::LISTEN_CONF", DEBUG_CCAPI20_Channel);
    ListenConf (Controller, Info);
}

/*===========================================================================*\
    CCAPI20_Channel - Ind-Messages
\*===========================================================================*/

tBool CCAPI20_Channel::CONNECT_IND (c_word, c_dword PLCI, cp_connectind *Params) {
    dhead ("CCAPI20_Channel::CONNECT_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (GetState() == cs_Registered) {
        dassert (GetController() == IllegalController);
        dassert (GetPLCI() == IllegalPLCI);
        dassert (GetNCCI() == IllegalNCCI);
        SetState (cs_D_ConnectIndication);
        SetPLCI (PLCI);
        SetController (GetControllerFromPLCI (PLCI));
        ConnectInd (GetController(), Params);
        RETURN ('x', vTrue);
    }
    RETURN ('x', vFalse);
}

tBool CCAPI20_Channel::CONNECT_ACTIVE_IND (c_word, c_dword PLCI, c_struct ConnectedNumber) {
    dhead ("CCAPI20_Channel::CONNECT_ACTIVE_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if ((GetState() != cs_D_ConnectPending) || (PLCI != GetPLCI())) {
        RETURN ('x', vFalse);
    }
    SetState (cs_D_Connected);
    CONNECT_ACTIVE_RESP (GetNextMsgNr(), PLCI);
    BChannelTimer.CallTimer (BCHANNELTIMER_WAITTIME);
    ConnectActiveInd (ConnectedNumber);
    if (Initiator == vTrue) {
        Initiator = vFalse;
        CONNECT_B3_REQ (GetNextMsgNr(), GetPLCI(), NCPIout);
    }
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::CONNECT_B3_IND (c_word, c_dword NCCI, c_struct) {
    dhead ("CCAPI20_Channel::CONNECT_B3_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if ((GetState() != cs_D_Connected) || (GetPLCIFromNCCI (NCCI) != GetPLCI())) {
        RETURN ('x', vFalse);
    }
    dassert (GetNCCI() == IllegalNCCI);
    dassert (Initiator == vFalse);
    SetNCCI (NCCI);
    SetState (cs_B_ConnectPending);
    cp_connectb3resp ConnB3Params = { creject_Accept, NCPIout };
    CONNECT_B3_RESP (GetNextMsgNr(), GetNCCI(), &ConnB3Params);
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::CONNECT_B3_ACTIVE_IND (c_word, c_dword NCCI, c_struct NCPI) {
    dhead ("CCAPI20_Channel::CONNECT_B3_ACTIVE_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if ((GetState() != cs_B_ConnectPending) || (NCCI != GetNCCI())) {
        RETURN ('x', vFalse);
    }
    dassert (GetPLCI() != IllegalPLCI);
    dwarning (Initiator == vFalse);
    SetState (cs_Connected);
    CONNECT_B3_ACTIVE_RESP (GetNextMsgNr(), NCCI);
    BChannelTimer.CancelTimer();
    ConnectB3ActiveInd (NCPI);
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::DATA_B3_IND (c_word, c_dword NCCI, cp_datab3 *Params) {
    dhead ("CCAPI20_Channel::DATA_B3_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (NCCI != GetNCCI()) {
        RETURN ('x', vFalse);
    }
    dassert (GetState() == cs_Connected);
    DataB3Ind (Params);
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::DISCONNECT_B3_IND (c_word, c_dword NCCI, cp_disconnectb3ind *Params) {
    dhead ("CCAPI20_Channel::DISCONNECT_B3_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (NCCI != GetNCCI()) {
        RETURN ('x', vFalse);
    }
    dassert ((GetState() == cs_B_ConnectPending) || (GetState() == cs_Connected) ||
             (GetState() == cs_B_DisconnectPending));
    dassert (GetPLCI() == GetPLCIFromNCCI (NCCI));
    DISCONNECT_B3_RESP (GetNextMsgNr(), NCCI);
    SetNCCI (IllegalNCCI);
    SetState (cs_B_Disconnected);
    DisconnectB3Ind (Params);
    if ((GetState() > cs_Registered) && (GetState() < cs_D_DisconnectPending)) {
        DISCONNECT_REQ (GetNextMsgNr(), GetPLCI());
    }
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::DISCONNECT_IND (c_word, c_dword PLCI, c_info Reason) {
    dhead ("CCAPI20_Channel::DISCONNECT_IND", DEBUG_CCAPI20_Channel);
    dparams ("s=%x", GetState());
    if (PLCI != GetPLCI()) {
        RETURN ('x', vFalse);
    }
    dassert (GetState() > cs_Registered);
    dwarning (GetNCCI() == IllegalNCCI);
    FreePLCIEntry (PLCI);
    DISCONNECT_RESP (GetNextMsgNr(), PLCI);
    BChannelTimer.CancelTimer();
    SetController (IllegalController);
    SetPLCI (IllegalPLCI);
    SetNCCI (IllegalNCCI);
    SetState (cs_Registered);
    Initiator = vFalse;
    DisconnectInd (Reason);
    RETURN ('x', vTrue);
}

tBool CCAPI20_Channel::INFO_IND (c_word, c_dword Address, cp_infoind *Params) {
    dhead ("CCAPI20_Channel::INFO_IND", DEBUG_CCAPI20_Channel);
    dassert (Params != 0);
    // TODO: Handle InfoInd for Controller-Addresses
    if (Address != GetPLCI()) {
        RETURN ('x', vFalse);
    }
    INFO_RESP (GetNextMsgNr(), GetControllerFromPLCI (Address));
    InfoInd (Params);
    RETURN ('x', vTrue);
}

/*===========================================================================*\
\*===========================================================================*/
