/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "Channel.h"
#include "ConvCapi.h"
#include "osImport.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define CAPI_CALLEDPARTYNUM_MAXLEN          70

/*===========================================================================*\
    CTransferChannel - Konstruktor
\*===========================================================================*/

CTransferChannel::CTransferChannel (void) : SendBlock (CAPI_MAX_OUTSTANDING_FRAMES) {
    dhead ("CTransferChannel-Constructor", DEBUG_CTransferChannel);
    pMSNList        = new CCntrlMSNList;
    deleteMSNList   = vTrue;

    controller      = IllegalController;
    cipValue        = cipvalue_Undefined;
    B1Prot          = b1prot_HDLC;
    B2Prot          = b2prot_X75;
    B3Prot          = b3prot_Transparent;
    B1Config        = 0;
    B3Config        = 0;
    sMSN            = 0;
    CalledPartyNum  = 0;
    szCallingNumber = 0;
    DDINumberLen    = 0;
}


/*===========================================================================*\
    CTransferChannel - Destruktor
\*===========================================================================*/

CTransferChannel::~CTransferChannel (void) {
    dhead ("CTransferChannel-Destructor", DEBUG_CTransferChannel);
    dwarning (CurState() == cs_Released);
    if (deleteMSNList == vTrue) {
        if (pMSNList) {
            pMSNList->RemoveAll();
        }
        delete pMSNList;
        pMSNList = 0;
    }
    controller = IllegalController;
    cipValue   = cipvalue_Undefined;
    Conv_DuplicateStruct (&B1Config, 0);
    Conv_DuplicateStruct (&B3Config, 0);
    Conv_DuplicateStruct (&sMSN, 0);
    Conv_DuplicateStruct (&CalledPartyNum, 0);
    Conv_DuplicateString (&szCallingNumber, 0);
}


/*===========================================================================*\
    CTransferChannel::ListenAdd
\*===========================================================================*/

void CTransferChannel::ListenAdd (tUInt Controller, tUInt CIPMask) {
    dhead ("CTransferChannel::ListenAdd", DEBUG_CTransferChannel);
    tUChar *myNum = 0;
    ConvStruct_CallingNumber (&myNum, sMSN);
    ListenAdd (Controller, CIPMask, (myNum) ? 1 : 0, &myNum);
    Conv_DuplicateString (&myNum, 0);
}


/*===========================================================================*\
    CTransferChannel::ListenAdd
\*===========================================================================*/

void CTransferChannel::ListenAdd (tUInt Controller, tUInt CIPMask, tUInt MainNumLen, tUInt DDILen, tUChar *firstMSN, tUChar *lastMSN) {
    dhead ("CTransferChannel::ListenAdd", DEBUG_CTransferChannel);
    if (!pMSNList) {
        deleteMSNList = vTrue;
        pMSNList      = new CCntrlMSNList;
        if (!pMSNList) {
            dwarning (0);
            return;
        }
    }
    pMSNList->Add (Controller, 0, CIPMask, MainNumLen, DDILen);

    tUInt   count = 0;
    tUChar *NumList[2];
    if (firstMSN && *firstMSN) {
        NumList [count++] = firstMSN;
        if (lastMSN && *lastMSN) {
            NumList [count++] = lastMSN;
        }
        pMSNList->AddMSN (Controller, count, (tUChar **)&NumList);
    }

    CheckListen();
}


/*===========================================================================*\
    CTransferChannel::ListenRemove
\*===========================================================================*/

void CTransferChannel::ListenRemove (tUInt Controller) {
    dhead ("CTransferChannel::ListenRemove", DEBUG_CTransferChannel);
    if (Controller) {
        if (pMSNList) {
            pMSNList->Remove (Controller);
        }

        cp_listenreq newListenParams;
        memset (&newListenParams, '\0', sizeof (cp_listenreq));
        ListenReq (Controller, &newListenParams);
    } else if (pMSNList) {
        pMSNList->RemoveAll();
    }
    CheckListen();
}


/*===========================================================================*\
    CTransferChannel::ShareMSNList
\*===========================================================================*/

void CTransferChannel::ShareMSNList (CCntrlMSNList *pSharedList) {
    dhead ("CTransferChannel::ShareMSNList", DEBUG_CTransferChannel);
    pMSNList      = pSharedList;
    deleteMSNList = (pMSNList) ? vFalse : vTrue;
}


/*===========================================================================*\
    CTransferChannel::MakeCall
\*===========================================================================*/

void CTransferChannel::MakeCall (tUShort Controller, tUChar *pTelNumber, c_cipvalue CipValue) {
    dhead ("CTransferChannel::MakeCall", DEBUG_CTransferChannel);
    dassert (pTelNumber != 0);

    if (GetState() <= cs_Registered) {
        DDINumberLen = 0;
        ConvUnion_CalledNumber (&CalledPartyNum, pTelNumber);
        if (CalledPartyNum) {
            controller       = Controller;
            cipValue         = CipValue;
            switch (GetState()) {
            case cs_Released:
            case cs_Releasing:
                Register();
                break;
            case cs_Registering:
                /*----- Nothing To Do! -----*/
                break;
            default:
                dassert (0);
                /*----- no Break! -----*/
            case cs_Registered:
                {
                    DoConnecting();
                    cp_connectreq ConParams = {cipValue, CalledPartyNum, sMSN, {B1Prot, B2Prot, B3Prot, B1Config, 0, B3Config}};
                    ConnectReq (controller, &ConParams);
                }
                break;
            }
            return;
        }
    }
    ConvUnion_CalledNumber (&CalledPartyNum, 0);
    IsDisconnected (ci_NoError);
}


/*===========================================================================*\
    CTransferChannel::AnswerCall
\*===========================================================================*/

void CTransferChannel::AnswerCall (tBool Accept) {
    dhead ("CTransferChannel::AnswerCall", DEBUG_CTransferChannel);
    if (GetState() == cs_D_ConnectIndication) {
        DDINumberLen = 0;
        if (Accept == vTrue) {
            cp_connectresp ConParams = {creject_Accept, sMSN, {B1Prot, B2Prot, B3Prot, B1Config, 0, B3Config}};
            ConnectResp (&ConParams);
        } else  {
            ConnectResp (&cpV_ConnectResp_Ignore);
        }
    }
}


/*===========================================================================*\
    CTransferChannel::PutData
\*===========================================================================*/

tBool CTransferChannel::PutData (tUByte *Data, tUShort DataLength, void *hDataID) {
    dhead ("CTransferChannel::PutData", DEBUG_CTransferChannel);
    dparams ("d=%p,l=%x,h=%x", Data, DataLength, hDataID);
    tUShort CapiHandle;
    if (SendBlock.New (hDataID, &CapiHandle) == vFalse) {
        //PutDataComplete (hDataID, i_QueueFull);
        RETURN ('x', vFalse);
    }
    dassert (CapiHandle <= MAXVAL_tUShort);
    cp_datab3 DataParams = {Data, DataLength, CapiHandle, 0};
    DataB3Req (&DataParams);
    RETURN ('x', vTrue);
}


/*===========================================================================*\
    CTransferChannel::Register_Complete
\*===========================================================================*/

void CTransferChannel::Register_Complete (c_info ErrorCode) {
    dhead ("CTransferChannel::Register_Complete", DEBUG_CTransferChannel);
    dwarning (ErrorCode == ci_NoError);
    if (CapiInfoIsSuccess (ErrorCode)) {
        if (CalledPartyNum) {
            DoConnecting();
            cp_connectreq ConParams = {cipValue, CalledPartyNum, sMSN, {B1Prot, B2Prot, B3Prot, B1Config, 0, B3Config}};
            ConnectReq (controller, &ConParams);
        }
        CheckListen();
    } else {
        if (CalledPartyNum) {
            ConvUnion_CalledNumber (&CalledPartyNum, 0);
            IsDisconnected (ErrorCode);
        }
        if (pMSNList && (pMSNList->IsEmpty() == vFalse)) {
            ListenError (0, ErrorCode);
        }
        IsReleased (ci_NoError);
    }
}


/*===========================================================================*\
    CTransferChannel::Release_Complete
\*===========================================================================*/

void CTransferChannel::Release_Complete (c_info ErrorCode) {
    dhead ("CTransferChannel::Release_Complete", DEBUG_CTransferChannel);
    IsReleased (ErrorCode);
}


/*===========================================================================*\
    CTransferChannel::DataB3Conf
\*===========================================================================*/

void CTransferChannel::DataB3Conf (cp_datab3conf *Params) {
    dhead ("CTransferChannel::DataB3Conf", DEBUG_CTransferChannel);
    dassert (Params != 0);
    void *hDataID;
    if (SendBlock.Free (Params->DataHandle, &hDataID) == vTrue) {
        PutDataComplete (hDataID, Params->Info);
    } else {
        dprintErr ("Great CAPI Error (Now disconnecting)!!!\n");
        Disconnect();
        dassert (0);
    }
}


/*===========================================================================*\
    CTransferChannel::ListenConf
\*===========================================================================*/

void CTransferChannel::ListenConf (c_dword Controller, c_info Info) {
    dhead ("CTransferChannel::ListenConf", DEBUG_CTransferChannel);
    dparams ("c=%x,i=%x", Controller, Info);
    if (CapiInfoIsError (Info) == vTrue) {
        //if (Info == ci_Cod_IllegalAddress) {
        //    // illegal controller-number can be deleted
        //    MSNList.Remove (Controller);
        //    CheckListen();
        //}
        ListenError (Controller, Info);
    }
}


/*===========================================================================*\
    CTransferChannel::CheckListen
\*===========================================================================*/

void CTransferChannel::CheckListen (void) {
    dhead ("CTransferChannel::CheckListen", DEBUG_CTransferChannel);
    tUInt        newController;
    cp_listenreq newListenParams;
    newListenParams.CallingPartyNumber = 0;

    dprint ("state=%x ", GetState());
    switch (GetState()) {
    case cs_Released:
    case cs_Releasing:
        if (pMSNList && (pMSNList->IsEmpty() == vFalse)) {
            Register();
        }
        break;
    case cs_Registering:
        // nothing to do!
        break;
    case cs_Registered:
        if (!pMSNList || (pMSNList->IsEmpty() == vTrue)) {
            Release();
            break;
        }
        // no break!
    default:
        if (pMSNList) {
            pMSNList->ResetGetNextMask();
            while (pMSNList->GetNextMask (&newController, &newListenParams.InfoMask, &newListenParams.CIPMask) == vTrue) {
                ListenReq (newController, &newListenParams);
            }
        }
        break;
    }
}


/*===========================================================================*\
    CTransferChannel::ConnectInd
\*===========================================================================*/

void CTransferChannel::ConnectInd (c_dword Controller, cp_connectind *Params) {
    dhead ("CTransferChannel::ConnectInd", DEBUG_CTransferChannel);
    if (pMSNList == 0) {
        ConnectResp (&cpV_ConnectResp_Ignore);
        return;
    }
    DDINumberLen = 0;
    switch (pMSNList->FindCallInfo (Controller, Params->CIPValue, Params->CalledPartyNumber)) {
    case CCntrlMSNList::i_DDICall:
        DDINumberLen = pMSNList->GetDDINumberLen (Controller);

        dassert (DDINumberLen < 254);
        dassert (DDINumberLen <= CAPI_CALLEDPARTYNUM_MAXLEN);
        if (DDINumberLen) {
            Conv_DuplicateStruct (&CalledPartyNum, 0);
            CalledPartyNum = new c_byte [CAPI_CALLEDPARTYNUM_MAXLEN + 2];
            if (CalledPartyNum) {
                ConvStruct_CallingNumber (&szCallingNumber, Params->CallingPartyNumber);
                controller        = Controller;
                cipValue          = Params->CIPValue;
                CalledPartyNum[0] = 1;
                CalledPartyNum[1] = 0x80;
                return;
            }
            dwarning (0);
        }
        dwarning (0);
        // no break!
    case CCntrlMSNList::i_AcceptCall:
        if (AcceptIncomingCall() == vTrue) {
            controller  = Controller;
            cipValue    = Params->CIPValue;
            tUChar *szN = 0;
            ConvStruct_CalledNumber (&szN, Params->CalledPartyNumber);
            ConvStruct_CallingNumber (&szCallingNumber, Params->CallingPartyNumber);
            dprint ("(%s)(%s)\n", szCallingNumber, szN);
            IncomingCall (Controller, cipValue, szCallingNumber, szN);
            Conv_DuplicateString (&szN, 0);
        }
        break;
    default:
        ConnectResp (&cpV_ConnectResp_Ignore);
        break;
    }
}


/*===========================================================================*\
    CTransferChannel::ConnectActiveInd
\*===========================================================================*/

void CTransferChannel::ConnectActiveInd (c_struct) {
    dhead ("CTransferChannel::ConnectActiveInd", DEBUG_CTransferChannel);
    //tUInt        newController;
    //tUInt        newInfoMask;
    //tUInt        newCIPMask;
    //cp_listenreq newListenParams = {0};
    //MSNList.ResetGetNextMask();
    //while (MSNList.GetNextMask (&newController, &newInfoMask, &newCIPMask) == vTrue) {
    //    ListenReq (newController, &newListenParams);
    //}

    ConnectProceeding();
}


/*===========================================================================*\
    CTransferChannel::ConnectB3ActiveInd
\*===========================================================================*/

void CTransferChannel::ConnectB3ActiveInd (c_struct NCPI) {
    dhead ("CTransferChannel::ConnectB3ActiveInd", DEBUG_CTransferChannel);
    cu_NCPI *NCPIcapi = 0;

    ConvStruct_NCPI (&NCPIcapi, NCPI, B3Prot);
    IsConnected(NCPIcapi);
    ConvStruct_NCPI (&NCPIcapi, 0, b3prot_Transparent);
}


/*===========================================================================*\
    CTransferChannel::DataB3Ind
\*===========================================================================*/

void CTransferChannel::DataB3Ind (cp_datab3 *Params) {
    dhead ("CTransferChannel::DataB3Ind", Dbg_Level_Infos); //DEBUG_CTransferChannel);
    dassert (Params != 0);
    GetData (Params->Data, Params->DataLength, Params->DataHandle);
}


/*===========================================================================*\
    CTransferChannel::DisconnectB3Ind
\*===========================================================================*/

void CTransferChannel::DisconnectB3Ind (cp_disconnectb3ind *Params) {
    dhead ("CTransferChannel::DisconnectB3Ind", DEBUG_CTransferChannel);
    dassert (Params != 0);
    ResetSendBlock();
    cu_NCPI *pNCPI = 0;
    ConvStruct_NCPI (&pNCPI, Params->NCPI, B3Prot);
    DoDisconnecting (Params->ReasonB3, pNCPI);
}


/*===========================================================================*\
    CTransferChannel::DisconnectInd
\*===========================================================================*/

void CTransferChannel::DisconnectInd (c_info Reason) {
    dhead ("CTransferChannel::DisconnectInd", DEBUG_CTransferChannel);
    controller = IllegalController;
    cipValue   = cipvalue_Undefined;
    Conv_DuplicateStruct (&CalledPartyNum, 0);
    Conv_DuplicateString (&szCallingNumber, 0);
    ResetSendBlock();
    if (DDINumberLen) {
        DDINumberLen = 0;
    } else {
        IsDisconnected (Reason);
    }
    CheckListen();
}


/*===========================================================================*\
    CTransferChannel::InfoInd
\*===========================================================================*/

void CTransferChannel::InfoInd (cp_infoind *Params) {
    dhead ("CTransferChannel::InfoInd", DEBUG_CTransferChannel);
    dparams ("%x", Params->InfoNumber);
    dassert (Params != 0);
    c_byte *Data;
    c_word  DataLen = GetBeginAndLenOfStructData (Params->InfoElement, &Data);
    dassert ((DataLen == 0) || (Data != 0));
    // at the moment we only used this call for DDI
    switch (Params->InfoNumber) {
    case 0x70: // called party number info element
        {
            if ((GetState() != cs_D_ConnectIndication) || (DataLen <= 1) || (IsCapiStructEmpty (CalledPartyNum) == vTrue)) {
                break;
            }
            Data++;
            DataLen--;
            dassert (CalcCapiStructDataLen (CalledPartyNum) >= 1);
            c_word NumLen = CalcCapiStructDataLen (CalledPartyNum) - 1;
            dassert (NumLen <= CAPI_CALLEDPARTYNUM_MAXLEN);
            dassert (CAPI_CALLEDPARTYNUM_MAXLEN < 254);
            if (DataLen >= CAPI_CALLEDPARTYNUM_MAXLEN) {
                Data   += DataLen - CAPI_CALLEDPARTYNUM_MAXLEN;
                DataLen = CAPI_CALLEDPARTYNUM_MAXLEN;
                NumLen  = 0;
            } else if (DataLen + NumLen > CAPI_CALLEDPARTYNUM_MAXLEN) {
                tUByte *dst = CalledPartyNum + 2;
                tUByte *src = dst + CAPI_CALLEDPARTYNUM_MAXLEN - DataLen;
                for (tUInt i = NumLen + DataLen - CAPI_CALLEDPARTYNUM_MAXLEN; i > 0; i--) {
                    *dst++ = *src++;
                }
                NumLen = CAPI_CALLEDPARTYNUM_MAXLEN - DataLen;
            }
            memcpy (CalledPartyNum + NumLen + 2, Data, DataLen);
            CalledPartyNum[0] = NumLen + DataLen + 1;
            if (CalledPartyNum[0] <= DDINumberLen) {
                break;
            }
        }
        // no break!
    case 0xA1: // sending complete info element
        if (GetState() == cs_D_ConnectIndication) {
            DDINumberLen = 0;
            tUInt offset = 0;
            if (  (pMSNList != 0)
               && (pMSNList->TestDDINumber (controller, CalledPartyNum, &offset) == CCntrlMSNList::i_AcceptCall)) {
                if (AcceptIncomingCall() == vTrue) {
                    dprint ("GetDDIOffset=%x ", offset);
                    tUChar *TelNumber = 0;
                    if (offset + 1 < CalledPartyNum[0]) {
                        tUInt len = CalledPartyNum[0] - offset - 1;
                        TelNumber = new tUChar [len + 1];
                        dwarning (TelNumber != 0);
                        if (TelNumber) {
                            memcpy (TelNumber, CalledPartyNum + offset + 2, len);
                            TelNumber[len] = '\0';
                        }
                    }
                    IncomingCall (controller, cipValue, szCallingNumber, TelNumber);
                    delete [] TelNumber;
                }
                break;
            }
            Conv_DuplicateStruct (&CalledPartyNum, 0);
            ConnectResp (&cpV_ConnectResp_Ignore);
        }
        break;
    }
}


/*===========================================================================*\
    CTransferChannel::ResetSendBlock
\*===========================================================================*/

void CTransferChannel::ResetSendBlock (void) {
    dhead ("CTransferChannel::ResetSendBlock", DEBUG_CTransferChannel);
    void *DelHandle;

    while (SendBlock.DelNextEntry (&DelHandle) == vTrue) {
        PutDataComplete (DelHandle, ci_Net_UnspecifiedCallClearing);
    }
}


/*===========================================================================*\
\*===========================================================================*/
