/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "DataHand.h"
#include "osImport.h"

/*===========================================================================*\
    CDataHandleAdmin - Constructor
\*===========================================================================*/

CDataHandleAdmin::CDataHandleAdmin (tUShort MaxEntries) {
    dhead ("CDataHandleAdmin-Constructor", DEBUG_CTransferChannel);
    dassert (MaxEntries > 0);

    nextOne    = 0;
    curEntries = 0;
    maxEntries = 0;
    Array      = new HandleArray [MaxEntries];
    Protect.Begin();
    if (Array) {
        Array[MaxEntries - 1].next = 0;
        for (tUShort i = (tUShort)(MaxEntries - 1); i > 0; i--) {
            Array[i - 1].next = (tUShort) (i + 1);
        }
        nextOne = 1;
        maxEntries = MaxEntries;
    }
    Protect.End();
}


/*===========================================================================*\
    CDataHandleAdmin - Destructor
\*===========================================================================*/

CDataHandleAdmin::~CDataHandleAdmin (void) {
    dhead ("CDataHandleAdmin-Destructor", DEBUG_CTransferChannel);
    Protect.Begin();
    HandleArray *tmpArray = Array;
    nextOne    = 0;
    curEntries = 0;
    maxEntries = 0;
    Array      = 0;
    Protect.End();
    delete [] tmpArray;
}


/*===========================================================================*\
    CDataHandleAdmin::New
\*===========================================================================*/

tBool CDataHandleAdmin::New (void *NewDataHandle, tUShort *RetHandle) {
    dhead ("CDataHandleAdmin::New", DEBUG_CTransferChannel);
    dwarning (NewDataHandle != 0);
    dassert (curEntries < maxEntries);
    dassert (RetHandle != 0);
    dassert (nextOne <= maxEntries);
    tBool fret = vFalse;

    Protect.Begin();
    if (nextOne) {
        tUShort get        = (tUShort)(nextOne - 1);
        nextOne            = Array[get].next;
        Array[get].next    = 0;
        Array[get].pointer = NewDataHandle;
        *RetHandle         = get;
        fret               = vTrue;
        curEntries++;
    }
    Protect.End();
    RETURN ('x', fret);
}


/*===========================================================================*\
    CDataHandleAdmin::Free
\*===========================================================================*/

tBool CDataHandleAdmin::Free (tUShort InHandle, void **RetDataHandle) {
    dhead ("CDataHandleAdmin::Free", DEBUG_CTransferChannel);
    dassert (RetDataHandle != 0);
    dassert (InHandle < maxEntries);
    dassert (curEntries > 0);
    dassert (nextOne <= maxEntries);
    dassert (Array[InHandle].pointer != 0);
    dassert (Array[InHandle].next == 0);

    Protect.Begin();
    *RetDataHandle          = Array[InHandle].pointer;
    Array[InHandle].pointer = 0;
    Array[InHandle].next    = nextOne;
    nextOne                 = (tUShort)(InHandle + 1);
    curEntries--;
    Protect.End();

    return (RetDataHandle) ? vTrue : vFalse;
}


/*===========================================================================*\
    CDataHandleAdmin::DelNextEntry
\*===========================================================================*/

tBool CDataHandleAdmin::DelNextEntry (void **HandleToDel) {
    dhead ("CDataHandleAdmin::DelNextEntry", DEBUG_CTransferChannel);
    dassert (nextOne <= maxEntries);
    tBool fret = vFalse;

    Protect.Begin();
    if (curEntries) {
        tUShort lauf;
        for (tUShort i = 1; i <= maxEntries; i++) {
            lauf = nextOne;
            while (lauf != 0) {
                if (lauf == i) {
                    break;
                } else {
                    lauf = Array[lauf - 1].next;
                    dassert (lauf <= maxEntries);
                    dinfo (lauf <= maxEntries, "lauf=%x ", lauf);
                }
            }
            if (lauf == 0) {
                if (HandleToDel) {
                    dwarning (Array[i - 1].pointer != 0);
                    *HandleToDel = Array[i - 1].pointer;
                }
                Array[i - 1].pointer = 0;
                Array[i - 1].next    = nextOne;
                nextOne              = i;
                fret                 = vTrue;
                curEntries--;
                break;
            }
        }
    }
    Protect.End();

    RETURN ('x', fret);
}


/*===========================================================================*\
\*===========================================================================*/
