/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "MSNList.h"
#include "ConvCapi.h"
#include "DynArray.h"
#include "osImport.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

class CControllerInfo : public CSortPListElement {
public:
    CControllerInfo (void);
    virtual ~CControllerInfo (void);

    void DelAllMSNs (void);
    tCompare Compare (void *RefCompData);
    void *GetRefCompData (void);

    tUInt         Controller;
    tUInt         InfoMask;
    tUInt         CIPMask;
    tUInt         MainNumLen;
    tUInt         DDILen;
    CDynamicArray MSNList;
};

/*===========================================================================*\
\*===========================================================================*/

tResult ArrayFindFunc_MSN (const void *data, const void *ItemFrag) {
    dhead ("ArrayFindFunc_MSN", DEBUG_CCntrlMSNList);
    dparams ("%x,%x", data, ItemFrag);
    dassert (data != 0);
    dassert (ItemFrag != 0);
    switch (Conv_CompareCalledPartyNumber (*(c_struct *)data, (c_struct)ItemFrag)) {
    case vEqual:
        RETURN ('x', vYes);
    case vHigher:
        RETURN ('x', vCancel);
    case vLower:
        break;
    }
    RETURN ('x', vNo);
}

/*===========================================================================*\
\*===========================================================================*/

tBool CCntrlMSNList::Set (tUInt Controller, tUInt InfoMask, tUInt CIPMask, tUInt MainNumLen, tUInt DDILen) {
    dhead ("CCntrlMSNList::Set", DEBUG_CCntrlMSNList);
    dassert (Controller > 0);
    if ((MainNumLen > 0) || (DDILen > 0)) {
        InfoMask |= infomask_CalledPartyNum | infomask_SendingComplete;
    }
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    if (!pE) {
        dprint ("New ControllerInfo! ");
        if (!CIPMask && !InfoMask) {
            return vTrue;
        }
        pE = new CControllerInfo;
        if (!pE) {
            return vFalse;
        }
        pE->Controller = Controller;
        CSortPointerList::Add (pE);
    }
    pE->CIPMask    = CIPMask;
    pE->InfoMask   = InfoMask;
    pE->MainNumLen = MainNumLen;
    pE->DDILen     = DDILen;
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

tBool CCntrlMSNList::Add (tUInt Controller, tUInt InfoMask, tUInt CIPMask, tUInt MainNumLen, tUInt DDILen) {
    dhead ("CCntrlMSNList::Add", DEBUG_CCntrlMSNList);
    dassert (Controller > 0);
    if ((MainNumLen > 0) || (DDILen > 0)) {
        InfoMask |= infomask_CalledPartyNum | infomask_SendingComplete;
    }
    if (!CIPMask && !InfoMask) {
        return vTrue;
    }
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    if (!pE) {
        dprint ("New ControllerInfo! ");
        pE = new CControllerInfo;
        if (!pE) {
            return vFalse;
        }
        pE->Controller = Controller;
        CSortPointerList::Add (pE);
    }
    pE->CIPMask  |= CIPMask;
    pE->InfoMask |= InfoMask;
    if (MainNumLen) {
        pE->MainNumLen = MainNumLen;
    }
    if (DDILen) {
        pE->DDILen = DDILen;
    }
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

CCntrlMSNList::ctInfo CCntrlMSNList::SetMSN (tUInt Controller, tUInt countMSNs, tUChar **MSNs) {
    dhead ("CCntrlMSNList::SetMsn", DEBUG_CCntrlMSNList);
    dparams ("%x,%x,%s", Controller, countMSNs, *MSNs);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    if (pE) {
        pE->DelAllMSNs();
    }
    return AddMSN (Controller, countMSNs, MSNs);
}

/*===========================================================================*\
\*===========================================================================*/

CCntrlMSNList::ctInfo CCntrlMSNList::AddMSN (tUInt Controller, tUInt countMSNs, tUChar **MSNs) {
    dhead ("CCntrlMSNList::AddMsn", DEBUG_CCntrlMSNList);
    dparams ("%x,%x,%s", Controller, countMSNs, *MSNs);
    dassert (Controller > 0);
    dassert (countMSNs > 0);
    ctInfo           fret = i_Okay;
    CControllerInfo *pE   = (CControllerInfo *)Find ((void *)Controller);
    if (!pE) {
        pE = new CControllerInfo;
        if (!pE) {
            RETURN ('x', i_Err_MemoryFull);
        }
        pE->Controller = Controller;
        CSortPointerList::Add (pE);
    }
    // MSNs are added to a sortedArray
    c_struct newMSN = 0;
    tUInt fPos;
    if (pE->InfoMask & infomask_CalledPartyNum) {
        // handling for DDI
        c_struct inMSN;
        fPos = pE->MSNList.CountUsedItems();
        dassert (fPos <= 2);
        dassert (countMSNs <= 2);
        dassert (pE->MSNList.ExistEmptyItems() == vFalse);
        ConvUnion_CalledNumber (&newMSN, *MSNs);
        if (!newMSN) {
            RETURN ('x', i_Err_MemoryFull);
        }
        if ((pE->MSNList.Get (0, &inMSN) == vFalse) || (Conv_CompareCalledPartyNumber (inMSN, newMSN) == vHigher)) {
            if (pE->MSNList.Set (0, newMSN) == 0) {
                // TODO?: DelAllMSNs();
                dwarning (0);
                ConvUnion_CalledNumber (&newMSN, 0);
                RETURN ('x', i_Err_MemoryAccess);
            }
            newMSN = 0;
        } else {
            ConvUnion_CalledNumber (&newMSN, 0);
        }
        if (countMSNs > 1) {
            ConvUnion_CalledNumber (&newMSN, *MSNs);
            if (!newMSN) {
                RETURN ('x', i_Err_MemoryFull);
            }
            if ((pE->MSNList.Get (1, &inMSN) == vFalse) || (Conv_CompareCalledPartyNumber (inMSN, newMSN) == vLower)) {
                if (pE->MSNList.Set (1, newMSN) == 0) {
                    // TODO?: DelAllMSNs();
                    dwarning (0);
                    ConvUnion_CalledNumber (&newMSN, 0);
                    RETURN ('x', i_Err_MemoryAccess);
                }
            } else {
                ConvUnion_CalledNumber (&newMSN, 0);
            }
        }
    } else {
        for (tUInt i = 0; i < countMSNs; i++, MSNs++) {
            dassert (*MSNs != 0);
            newMSN = 0;
            ConvUnion_CalledNumber (&newMSN, *MSNs);
            if (!newMSN) {
                dwarning (0);
                fret = i_Wrn_NotAllAdded;
                continue;
            }
            switch (pE->MSNList.Find (ArrayFindFunc_MSN, newMSN, 0, &fPos)) {
            case vNo:
                // Add at the end
                fPos = pE->MSNList.CountAllItems();
                /*----- no Break! -----*/
            case vCancel:
                dprint ("InsPos=%x ", fPos);
                if (pE->MSNList.Ins (fPos, &newMSN) == vTrue) {
                    break;
                }
            default:
                dwarning (0);
                Conv_DuplicateStruct (&newMSN, 0);
                break;
            }
        }
    }
    RETURN ('x', fret);
}

/*===========================================================================*\
\*===========================================================================*/

void CCntrlMSNList::Remove (tUInt Controller) {
    dhead ("CCntrlMSNList::Remove", DEBUG_CCntrlMSNList);
    dparams ("%x", Controller);
    dassert (Controller > 0);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    if (pE) {
        delete pE;
    }
}

/*===========================================================================*\
\*===========================================================================*/

CCntrlMSNList::ctInfo CCntrlMSNList::FindCallInfo (tUInt Controller, c_cipvalue CIPValue, c_struct MSN) {
    dhead ("CCntrlMSNList::FindCallInfo", DEBUG_CCntrlMSNList);
    dparams ("%x,%x", Controller, CIPValue);
    dassert (Controller > 0);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    dassert (pE != 0);
    if (pE && ((1 << CIPValue) & pE->CIPMask)) {
        if (pE->InfoMask & infomask_CalledPartyNum) {
            RETURN ('x', i_DDICall);
        }
        if (  (IsCapiStructEmpty (MSN) == vTrue)
           || (pE->MSNList.ExistUsedItems() == vFalse)
           || (pE->MSNList.Find (ArrayFindFunc_MSN, MSN) == vYes)) {
            RETURN ('x', i_AcceptCall);
        }
    }
    RETURN ('x', i_RejectCall);
}

/*===========================================================================*\
\*===========================================================================*/

CCntrlMSNList::ctInfo CCntrlMSNList::TestDDINumber (tUInt Controller, c_struct DDINumber, tUInt *pMainNumLen) {
    dhead ("CCntrlMSNList::TestDDINumber", DEBUG_CCntrlMSNList);
    dassert (Controller > 0);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    dassert (pE != 0);
    if (pE) {
        if (pMainNumLen) {
            *pMainNumLen = pE->MainNumLen;
        }
        dassert (pE->InfoMask & infomask_CalledPartyNum);
        dassert (pE->MSNList.CountUsedItems() <= 2);
        c_struct inMSN;
        if ((IsCapiStructEmpty (DDINumber) == vTrue) || (pE->MSNList.Get (0, &inMSN) == vFalse)) {
            RETURN ('x', i_AcceptCall);
        }
        switch (Conv_CompareCalledPartyNumber (inMSN, DDINumber)) {
        case vLower:
            if (  (pE->MSNList.Get (1, &inMSN) == vFalse)
               || (Conv_CompareCalledPartyNumber (inMSN, DDINumber) == vLower)) {
                break;
            }
            // no break;
        case vEqual:
            RETURN ('x', i_AcceptCall);
        case vHigher:
            break;
        }
    }
    RETURN ('x', i_RejectCall);
}

/*===========================================================================*\
\*===========================================================================*/

tBool CCntrlMSNList::GetMask (tUInt Controller, tUInt *pInfoMask, tUInt *pCIPMask) {
    dhead ("CCntrlMSNList::BuildMask", DEBUG_CCntrlMSNList);
    dassert (Controller != IllegalController);
    dassert (pCIPMask != 0);
    dassert (pInfoMask != 0);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    if (!pE) {
        return vFalse;
    }
    *pCIPMask  = pE->CIPMask;
    *pInfoMask = pE->InfoMask;
    //if (!*pCIPMask && !*pInfoMask) {
    //    pCurPos = (CControllerInfo *)pE->GetPrev();
    //    delete pE;
    //}
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

tBool CCntrlMSNList::GetNextMask (tUInt *pController, tUInt *pInfoMask, tUInt *pCIPMask) {
    dhead ("CCntrlMSNList::BuildNextMask", DEBUG_CCntrlMSNList);
    dassert (pController != 0);
    dassert (pCIPMask != 0);
    dassert (pInfoMask != 0);
    tBool fret          = vFalse;
    CControllerInfo *pE = pCurPos;
    pE = (CControllerInfo *)((pE != 0) ? pE->GetHigher() : GetLowest());
    for (; (pE != 0); pE = (CControllerInfo *)pE->GetHigher()) {
        if (pE->CIPMask || !pE->InfoMask) {
            *pController = pE->Controller;
            *pCIPMask    = pE->CIPMask;
            *pInfoMask   = pE->InfoMask;
            fret         = vTrue;
            dprint ("Cntrl=%x,Info=%x,CIP=%x ", *pController, *pInfoMask, *pCIPMask);
            break;
            //if (!*pCIPMask && !*pInfoMask) {
            //    pCurPos = pE->GetPrev();
            //    delete pE;
            //    return fret;
            //}
        }
    }
    pCurPos = pE;
    RETURN ('x', fret);
}

/*===========================================================================*\
\*===========================================================================*/

tUInt CCntrlMSNList::GetDDINumberLen (tUInt Controller) {
    dhead ("CCntrlMSNList::GetDDINumberLen", DEBUG_CCntrlMSNList);
    dassert (Controller > 0);
    CControllerInfo *pE = (CControllerInfo *)Find ((void *)Controller);
    RETURN ('x', (pE) ? (pE->MainNumLen + pE->DDILen) : 0);
}

/*===========================================================================*\
    CControllerInfo - Funktionen
\*===========================================================================*/

CControllerInfo::CControllerInfo (void) : MSNList (1, sizeof (c_struct)) {
    dhead ("CControllerInfo-Constructor", DEBUG_CCntrlMSNList);
    Controller = 0;
    InfoMask   = 0;
    CIPMask    = 0;
    MainNumLen = 0;
    DDILen     = 0;
}

CControllerInfo::~CControllerInfo (void) {
    dhead ("CControllerInfo-Destructor", DEBUG_CCntrlMSNList);
    DelAllMSNs();
}

void CControllerInfo::DelAllMSNs (void) {
    dhead ("CControllerInfo::DelAllMSNs", DEBUG_CCntrlMSNList);
    for (c_struct delMSN = 0; (MSNList.DelLast (&delMSN) == vTrue); delMSN = 0) {
        Conv_DuplicateStruct (&delMSN, 0);
    }
}

tCompare CControllerInfo::Compare (void *RefCompData) {
    dhead ("CControllerInfo::Compare", DEBUG_CCntrlMSNList);
    dparams ("%x,%x", Controller, RefCompData);
    return ((tUInt)RefCompData < Controller) ? vLower : ((tUInt)RefCompData == Controller) ? vEqual : vHigher;
}

void *CControllerInfo::GetRefCompData (void) {
    dhead ("CControllerInfo::GetRefCompData", DEBUG_CCntrlMSNList);
    return (void *)Controller;
}

/*===========================================================================*\
\*===========================================================================*/
