/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CAPI2DEF_H_
#define _CAPI2DEF_H_

#include "aTypes.h"
#include "osmem.h"

/*===========================================================================*\
\*===========================================================================*/

#define IllegalApplID                   0
#define IllegalController               0
#define IllegalPLCI                     0
#define IllegalNCCI                     0

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define CAPI_DEFAULT_S0_LINES_COUNT     2
#define CAPI_MAX_MESSAGE_SIZE           512
#define CAPI_MAX_DATA_SIZE              2048
#define CAPI_MAX_OUTSTANDING_FRAMES     7

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

/*----- basic-type definitions -----*/

typedef tUByte    c_byte;
typedef tUShort   c_word;
typedef tUInt     c_dword;
typedef tULong    c_qword;
typedef c_byte   *c_message;
typedef c_message c_struct;
typedef c_message c_dataptr;

/*----- errornumbers -----*/

typedef enum {
    ci_NoError                        = 0x0000,
    ci_Wrn_NCPIignored                = 0x0001,
    ci_Wrn_FlagsIgnored               = 0x0002,
    ci_Wrn_AlertAlreadySend           = 0x0003,
    ci_Wrn_HighestWarningValue        = 0x00FF,     // Special Warning Value
    ci_Reg_ToManyAppls                = 0x1001,
    ci_Reg_LogBlkSizeToSmall          = 0x1002,
    ci_Reg_BuffExeceeds64k            = 0x1003,
    ci_Reg_MsgBufSizeToSmall          = 0x1004,
    ci_Reg_NumLogConnNotSupported     = 0x1005,
    ci_Reg_Reserved                   = 0x1006,
    ci_Reg_Busy                       = 0x1007,
    ci_Reg_OSResourceErr              = 0x1008,
    ci_Reg_NotInstalled               = 0x1009,
    ci_Reg_CtrlerNotSupportExtEquip   = 0x100a,
    ci_Reg_CtrlerOnlySupportExtEquip  = 0x100b,
    ci_Msg_IllAppNr                   = 0x1101,
    ci_Msg_IllCmdOrSubcmdOrMsgToSmall = 0x1102,
    ci_Msg_SendQueueFull              = 0x1103,
    ci_Msg_ReceiveQueueEmpty          = 0x1104,
    ci_Msg_ReceiveOverflow            = 0x1105,
    ci_Msg_UnknownNotPar              = 0x1106,
    ci_Msg_Busy                       = 0x1107,
    ci_Msg_OSResourceErr              = 0x1108,
    ci_Msg_NotInstalled               = 0x1109,
    ci_Msg_CtrlerNotSupportExtEquip   = 0x110a,
    ci_Msg_CtrlerOnlySupportExtEquip  = 0x110b,
    ci_Cod_MsgNotSupportInCurState    = 0x2001,
    ci_Cod_IllegalAddress             = 0x2002,
    ci_Cod_OutOfPLCI                  = 0x2003,
    ci_Cod_OutOfNCCI                  = 0x2004,
    ci_Cod_OutOfListen                = 0x2005,
    ci_Cod_OutOfFax                   = 0x2006,
    ci_Cod_IllegalMsgParamCoding      = 0x2007,
    ci_Ser_B1ProtNotSupported         = 0x3001,
    ci_Ser_B2ProtNotSupported         = 0x3002,
    ci_Ser_B3ProtNotSupported         = 0x3003,
    ci_Ser_B1ProtParamNotSupported    = 0x3004,
    ci_Ser_B2ProtParamNotSupported    = 0x3005,
    ci_Ser_B3ProtParamNotSupported    = 0x3006,
    ci_Phy_ProtErrLayer1              = 0x3301,
    ci_Phy_ProtErrLayer2              = 0x3302,
    ci_Phy_ProtErrLayer3              = 0x3303,
    ci_Phy_T30ConnectToNoFax          = 0x3311,
    ci_Net_UnspecifiedCallClearing    = 0x3400
} c_info;

/*----- rejectvalues -----*/

typedef enum {
    creject_Accept                    = 0,
    creject_Ignore                    = 1,
    creject_Reject                    = 2
} c_reject;

/*----- CIP Values -----*/

typedef enum {
    cipvalue_Undefined                = 0,
    cipvalue_Speech                   = 1,
    cipvalue_Digital                  = 2,
    cipvalue_RestrictedDigital        = 3,
    cipvalue_3_1kHzAudio              = 4,
    cipvalue_7kHzAudio                = 5,
    cipvalue_Video                    = 6,
    cipvalue_PacketMode               = 7,
    cipvalue_56kBitRateAdaption       = 8,
    cipvalue_UnrestDigitalTones       = 9,
    cipvalue_Telephony                = 16,
    cipvalue_G3Fax                    = 17,
    cipvalue_G4Fax                    = 18,
    cipvalue_TeletexG4Fax             = 19,
    cipvalue_TeletexProcessable       = 20,
    cipvalue_TeletexBasicMode         = 21,
    cipvalue_Videotex                 = 22,
    cipvalue_Telex                    = 23,
    cipvalue_MessageHandlingSystems   = 24,
    cipvalue_OSIApplication           = 25,
    cipvalue_7kHzTelephony            = 26,
    cipvalue_VideoTelephony1          = 27,
    cipvalue_VideoTelephony2          = 28
} c_cipvalue;

/*----- CIP Masks -----*/

typedef enum {
    cipmask_Undefined                 = (1 << cipvalue_Undefined),
    cipmask_Speech                    = (1 << cipvalue_Speech),
    cipmask_Digital                   = (1 << cipvalue_Digital),
    cipmask_RestrictedDigital         = (1 << cipvalue_RestrictedDigital),
    cipmask_3_1kHzAudio               = (1 << cipvalue_3_1kHzAudio),
    cipmask_7kHzAudio                 = (1 << cipvalue_7kHzAudio),
    cipmask_Video                     = (1 << cipvalue_Video),
    cipmask_PacketMode                = (1 << cipvalue_PacketMode),
    cipmask_56kBitRateAdaption        = (1 << cipvalue_56kBitRateAdaption),
    cipmask_UnrestDigitalTones        = (1 << cipvalue_UnrestDigitalTones),
    cipmask_Telephony                 = (1 << cipvalue_Telephony),
    cipmask_G3Fax                     = (1 << cipvalue_G3Fax),
    cipmask_G4Fax                     = (1 << cipvalue_G4Fax),
    cipmask_TeletexG4Fax              = (1 << cipvalue_TeletexG4Fax),
    cipmask_TeletexProcessable        = (1 << cipvalue_TeletexProcessable),
    cipmask_TeletexBasicMode          = (1 << cipvalue_TeletexBasicMode),
    cipmask_Videotex                  = (1 << cipvalue_Videotex),
    cipmask_Telex                     = (1 << cipvalue_Telex),
    cipmask_MessageHandlingSystems    = (1 << cipvalue_MessageHandlingSystems),
    cipmask_OSIApplication            = (1 << cipvalue_OSIApplication),
    cipmask_7kHzTelephony             = (1 << cipvalue_7kHzTelephony),
    cipmask_VideoTelephony1           = (1 << cipvalue_VideoTelephony1),
    cipmask_VideoTelephony2           = (1 << cipvalue_VideoTelephony2)
} c_cipmask;

/*----- Info Masks -----*/

typedef enum {
    infomask_Cause                    = 0x0001,
    infomask_DateTime                 = 0x0002,
    infomask_Display                  = 0x0004,
    infomask_UserUser                 = 0x0008,
    infomask_CallProgress             = 0x0010,
    infomask_Facility                 = 0x0020,
    infomask_ChargeInfo               = 0x0040,
    infomask_CalledPartyNum           = 0x0080,
    infomask_ChannelInfo              = 0x0100,
    infomask_EarlyB3Connect           = 0x0200,
    infomask_RedirectInfo             = 0x0400,
    infomask_SendingComplete          = 0x1000
} c_infomask;


/*----- B - Protocols -----*/

typedef enum {
    b1prot_HDLC                       = 0,
    b1prot_Transparent                = 1,
    b1prot_V110_asynch                = 2,
    b1prot_V110_synch                 = 3,
    b1prot_G3Fax                      = 4,
    b1prot_invertedHDLC               = 5,
    b1prot_56kBitTransparent          = 6,
    b1prot_Modem_negotiate            = 7,
    b1prot_Modem_asynch               = 8,
    b1prot_Modem_synch                = 9
} c_b1prot;

typedef enum {
    b2prot_X75                        = 0,
    b2prot_Transparent                = 1,
    b2prot_SDLC                       = 2,
    b2prot_LAPD                       = 3,
    b2prot_G3Fax                      = 4,
    b2prot_PPP                        = 5,
    b2prot_IgnoreFramingError         = 6,
    b2prot_Modem_negotiate            = 7,
    b2prot_X75_V42bis                 = 8,
    b2prot_V120_asynch                = 9,
    b2prot_V120_V42bis_asynch         = 10,
    b2prot_V120_transparent           = 11,
    b2prot_LAPD_SAPI                  = 12
} c_b2prot;

typedef enum {
    b3prot_Transparent                = 0,
    b3prot_T90NL                      = 1,
    b3prot_ISO8208                    = 2,
    b3prot_X25                        = 3,
    b3prot_G3Fax                      = 4,
    b3prot_G3Fax_extended             = 5,
  //b3prot_reserved               = 6,  Reserved == not usable => doesn't need to be defined
    b3prot_Modem                      = 7
} c_b3prot;

/*----- facility -----*/

typedef enum {
    facilselect_Handset               = 0,
    facilselect_DTMF                  = 1,
    facilselect_V42Compression        = 2,
    facilselect_SupplServices         = 3,
    facilselect_PMWakeup              = 4,
} c_facilselect;

typedef enum {
    supplserv_GetSupportedServices    = 0x0000,
    supplserv_Listen                  = 0x0001,
    supplserv_Hold                    = 0x0002,
    supplserv_Retrieve                = 0x0003,
    supplserv_Suspend                 = 0x0004,
    supplserv_Resume                  = 0x0005,
    supplserv_ECT                     = 0x0006,
    supplserv_3PTY_Begin              = 0x0007,
    supplserv_3PTY_End                = 0x0008,
    supplserv_CF_Activate             = 0x0009,
    supplserv_CF_Deactivate           = 0x000A,
    supplserv_CF_InterrogateParams    = 0x000B,
    supplserv_CF_InterrogateNumbers   = 0x000C,
    supplserv_CD                      = 0x000D,
    supplserv_MCID                    = 0x000E,
    supplserv_CCBS_Request            = 0x000F,
    supplserv_CCBS_Deactivate         = 0x0010,
    supplserv_CCBS_Interrogate        = 0x0011,
    supplserv_CCBS_Call               = 0x0012,
    // Supplementary Services Notfications
    supplserv_Hold_Notify             = 0x8000,
    supplserv_Retrieve_Notify         = 0x8001,
    supplserv_Suspend_Notify          = 0x8002,
    supplserv_Resume_Notify           = 0x8003,
    supplserv_CallIsDiverting_Notify  = 0x8004,
    supplserv_DiversionActive_Notify  = 0x8005,
    supplserv_CF_Activate_Notify      = 0x8006,
    supplserv_CF_Deactivate_Notify    = 0x8007,
    supplserv_Diversion_Information   = 0x8008,
    supplserv_CT_Alerted_Notify       = 0x8009,
    supplserv_CT_Active_Notify        = 0x800A,
    supplserv_ConferenceEstablished   = 0x800B,
    supplserv_ConferenceDisconnected  = 0x800C,
    supplserv_CCBS_EraseCallLinkID    = 0x800D,
    supplserv_CCBS_Status             = 0x800E,
    supplserv_CCBS_RemoteUserFree     = 0x800F,
    supplserv_CCBS_B_Free             = 0x8010,
    supplserv_CCBS_Erase              = 0x8011,
    supplserv_CCBS_StopAlerting       = 0x8012,
    supplserv_CCBS_InfoRetain         = 0x8013
} c_supplserv;

/*---------------------------------------------------------------------------*\
    CAPI function parameters
\*---------------------------------------------------------------------------*/
#pragma pack(1)

typedef struct _ctP_GetProfile {
    c_word  NumControllers;
    c_word  NumBChannels;
    c_dword GlobalOptions;
    c_dword B1ProtocolSupport;
    c_dword B2ProtocolSupport;
    c_dword B3ProtocolSupport;
    c_byte  Reserved[24];
    c_byte  ManufacturerSpecific[20];
} cp_getprofile;

typedef struct _ctP_BProtocol {
    c_b1prot B1Protocol;
    c_b2prot B2Protocol;
    c_b3prot B3Protocol;
    c_struct B1Configuration;
    c_struct B2Configuration;
    c_struct B3Configuration;
} cp_bprotocol;

typedef struct _ctP_ConnectReq {
    c_cipvalue   CIPValue;
    c_struct     CalledPartyNumber;
    c_struct     CallingPartyNumber;
    cp_bprotocol bp;
} cp_connectreq;

typedef struct _ctP_ConnectInd {
    c_cipvalue CIPValue;
    c_struct   CalledPartyNumber;
    c_struct   CallingPartyNumber;
} cp_connectind;

typedef struct _ctP_ConnectResp {
    c_reject     Reject;
    c_struct     ConnectedNumber;
    cp_bprotocol bp;
} cp_connectresp;

typedef struct _ctP_ConnectB3Resp {
    c_reject Reject;
    c_struct NCPI;
} cp_connectb3resp;

typedef struct _ctP_DataB3 {
    c_dataptr Data;
    c_word    DataLength;
    c_word    DataHandle;
    c_word    Flags;
} cp_datab3;

typedef struct _ctP_DataB3Conf {
    c_word DataHandle;
    c_info Info;
} cp_datab3conf;

typedef struct _ctP_DisconnectB3Ind {
    c_info   ReasonB3;
    c_struct NCPI;
} cp_disconnectb3ind;

typedef struct _ctP_Facility {
    c_facilselect FacilitySelector;
    c_struct      FacilityParam;
} cp_facility;

typedef struct _ctP_FacilityConf {
    c_facilselect FacilitySelector;
    c_struct      FacilityParam;
    c_info        Info;
} cp_facilityconf;

typedef struct _ctP_InfoReq {
    c_struct CalledPartyNumber;
    c_struct BChannelInformation;
    c_struct KeypadFacility;
    c_struct UserUserData;
    c_struct FacilityDataArray;
} cp_inforeq;

typedef struct _ctP_InfoInd {
    c_word   InfoNumber;
    c_struct InfoElement;
} cp_infoind;

typedef struct _ctP_ListenReq {
    c_dword  InfoMask;
    c_dword  CIPMask;
    c_struct CallingPartyNumber;
} cp_listenreq;

typedef struct _ctP_Manufacturer {
    c_dword  ManuID;
    c_dword  Class;
    c_dword  Function;
    c_struct ManuData;
} cp_manufacturer;

#pragma pack()

/*---------------------------------------------------------------------------*\
    CapiInfoIs... Makros
\*---------------------------------------------------------------------------*/

#define CapiInfoIsSuccess(info)  ((info) <= ci_Wrn_HighestWarningValue)
#define CapiInfoIsError(info)    ((info) > ci_Wrn_HighestWarningValue)
#define CapiInfoIsRegError(info) (((info) & 0xFF00) == 0x1000)

/*---------------------------------------------------------------------------*\
    CalcCapiStructLen function
\*---------------------------------------------------------------------------*/
#ifdef __cplusplus

inline tBool IsCapiStructEmpty (c_struct s) {
    return (s && *s) ? vFalse : vTrue;
}

inline c_word CalcCapiStructLen (c_struct s) {
    return (c_word)((s) ? ((s[0] == 255) ? get16 (s + 1) + 3 : (s[0] + 1)) : 1);
}

inline c_word CalcCapiStructDataLen (c_struct s) {
    return (c_word)((s) ? ((s[0] == 255) ? get16 (s + 1) : (s[0])) : 0);
}

inline c_byte *GetBeginOfStructData (c_struct s) {
    return (s) ? ((s[0] == 255) ? s + 3 : s + 1) : 0;
}

inline c_word GetBeginAndLenOfStructData (c_struct s, c_byte **pData) {
    // assert (pData != 0)
    if (!s) {
        return 0;
    } else if (s[0] == 255) {
        *pData = s + 3;
        return get16 (s + 1);
    }
    *pData = s + 1;
    return s[0];
}

#else

#define IsCapiStructEmpty(s) (((s) && *(s)) ? vFalse : vTrue)

#define CalcCapiStructLen(s) ((c_word)((s) ? (((s)[0] == 255) ? get16 ((s) + 1) + 3 : ((s)[0] + 1)) : 1))

#define CalcCapiStructDataLen(s) ((c_word)((s) ? (((s)[0] == 255) ? get16 ((s) + 1) : ((s)[0])) : 0))

#define GetBeginOfStructData(s) ((s) ? (((s)[0] == 255) ? (s) + 3 : (s) + 1) : 0)

#endif

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#endif
