/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _BYTEQUEU_H_
#define _BYTEQUEU_H_

#include "aTypes.h"
#include "Protect.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define ByteQueue_CountLocks        2


/*===========================================================================*\
    CStandardByteQueue
\*===========================================================================*/

class CStandardByteQueue {
public:
    CStandardByteQueue (tUInt maxQueueSize, tBool PartFillAllow = vFalse);
    virtual ~CStandardByteQueue (void);

    tUInt Put (tUChar *dataIn, tUInt length);
    tUInt PutInFront (tUChar *dataIn, tUInt length);
    tUInt Get (tUChar *dataOut, tUInt maxLength);
    void Clear (void);

    tBool IsEmpty (void);
    tUInt UsedSize (void);
    tUInt FreeSize (void);
    tUInt MaxSize (void);
    void  SetPartFill (tBool PartFillAllow);

    tBool Resize (tUInt newMaxQueueSize);

private:
    CProtectionLock ProtectPut;
    CProtectionLock ProtectGet;

    tUChar *buffer;
    tUInt   maxCount;
    tUInt   head;
    tUInt   tail;
    tBool   bPartFill;
};

/*===========================================================================*\
    inline Functionen
\*===========================================================================*/

inline tBool CStandardByteQueue::IsEmpty (void) {
    return (head == tail) ? vTrue : vFalse;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt CStandardByteQueue::UsedSize (void) {
    tUInt dHead = head;
    tUInt dTail = tail;

    if (dHead < dTail) {
        dHead += maxCount;
    }
    return (dHead - dTail);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt CStandardByteQueue::FreeSize (void) {
    if (!maxCount) {
        return 0;
    }
    tUInt dHead = head;
    tUInt dTail = tail;

    if (dHead >= dTail) {
        dTail += maxCount;
    }
    return (dTail - dHead - 1);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt CStandardByteQueue::MaxSize (void) {
    return (maxCount) ? (maxCount - 1) : 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CStandardByteQueue::SetPartFill (tBool PartFillAllow) {
    bPartFill = PartFillAllow;
}

/*===========================================================================*\
\*===========================================================================*/


#endif
