/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CSTRING_H_
#define _CSTRING_H_

#include "aTypes.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define CSTRING_RESIZE_ALLOCATE_OFFSET          100

typedef tUChar       tStringChar;
typedef tStringChar *tString;

/*===========================================================================*\
\*===========================================================================*/

class CDynamicString {
public:
    CDynamicString (void);
    virtual ~CDynamicString (void);

    tBool Set (tString string, tUInt len = 0);
    tBool Set (char *string, tUInt len = 0);
    tBool Set (CDynamicString *pcstr);
    tBool Append (tString string, tUInt len = 0);
    tBool Append (char *string, tUInt len = 0);
    tBool Append (CDynamicString *pcstr);

    tBool Change (tString string, tUInt startPos, tUInt len = 0);
    tBool Change (char *string, tUInt startPos, tUInt len = 0);
    tBool Change (CDynamicString *pcstr, tUInt startPos, tUInt len = 0);
    tBool Insert (tString string, tUInt startPos, tUInt len = 0);
    tBool Insert (char *string, tUInt startPos, tUInt len = 0);
    tBool Insert (CDynamicString *pcstr, tUInt startPos, tUInt len = 0);

    void Get (tString string, tUInt startPos = 0, tUInt len = 0);
    tStringChar GetChar (tUInt pos);
    tStringChar GetLastChar (void);
    tUInt FindChar (tStringChar ch, tUInt startpos = 0); // not found => return = MAXVAL_tUInt

    void  SetLen (tSInt newLen);   // newLen < 0 => value is an offset
    void  Remove (tUInt startPos, tUInt len);
    void  RemoveAll (void);
    tBool Resize (tUInt newMaxSize);

    tBool   IsEmpty (void);
    tUInt   GetLen (void);
    tUInt   GetSize (void);
    tUInt   GetMaxSize (void);
    tString GetPointer (void);
    tBool   DetectAnError (void);

private:
    tBool DynExpand (tUInt newMaxSize);

    tString pntr;
    tUInt   curLen;
    tUInt   maxSize;
    tBool   detectError;
};

/*===========================================================================*\
    inline fuctions from CDynamicString
\*===========================================================================*/

inline tBool CDynamicString::Set (tString string, tUInt len) {
    RemoveAll();
    return Append (string, len);
}

inline tBool CDynamicString::Set (char *string, tUInt len) {
    RemoveAll();
    return Append (string, len);
}

inline tBool CDynamicString::Set (CDynamicString *pcstr) {
    if (pcstr->GetLen()) {
        return Set (pcstr->GetPointer(), pcstr->GetLen());
    }
    RemoveAll();
    return vTrue;
}

inline tBool CDynamicString::Append (tString string, tUInt len) {
    return Change (string, GetLen(), len);
}

inline tBool CDynamicString::Append (char *string, tUInt len) {
    return Change (string, GetLen(), len);
}

inline tBool CDynamicString::Append (CDynamicString *pcstr) {
    return (pcstr->GetLen()) ? Change (pcstr->GetPointer(), GetLen(), pcstr->GetLen()) : vTrue;
}

inline tBool CDynamicString::Change (CDynamicString *pcstr, tUInt startPos, tUInt len) {
    return (pcstr->GetLen()) ? Change (pcstr->GetPointer(), startPos, (len) ? len : pcstr->GetLen()) : vTrue;
}

inline tBool CDynamicString::Insert (CDynamicString *pcstr, tUInt startPos, tUInt len) {
    return (pcstr->GetLen()) ? Insert (pcstr->GetPointer(), startPos, (len) ? len : pcstr->GetLen()) : vTrue;
}

inline tStringChar CDynamicString::GetChar (tUInt pos) {
    //assert (pos < GetLen());
    //assert (pntr != 0);
    return pntr[pos];
}

inline tStringChar CDynamicString::GetLastChar (void) {
    //assert (GetLen() > 0);
    //assert (pntr != 0);
    return pntr[GetLen() - 1];
}

inline void CDynamicString::SetLen (tSInt newLen) {
    // assert (pntr != 0);
    if (newLen < 0) {
        // assert (GetLen() > -1 * newLen)
        curLen += newLen;
    } else {
        // assert (newLen <= GetLen())
        curLen  = newLen;
    }
    pntr[curLen] = '\0';
}

inline void CDynamicString::RemoveAll (void) {
    curLen = 0;
    if (GetMaxSize()) {
        // assert (pntr != 0);
        pntr[0] = '\0';
    }
}

inline tBool CDynamicString::IsEmpty (void) {
    return (curLen == 0);
}

inline tUInt CDynamicString::GetLen (void) {
    return curLen;
}

inline tUInt CDynamicString::GetSize (void) {
    return (curLen) ? curLen + 1 : 0;
}

inline tUInt CDynamicString::GetMaxSize (void) {
    return maxSize;
}

inline tString CDynamicString::GetPointer (void) {
    return (curLen) ? pntr : 0;
}

inline tBool CDynamicString::DetectAnError (void) {
    if (detectError == vFalse) {
        return vFalse;
    }
    detectError = vFalse;
    return vTrue;
}

inline tBool CDynamicString::DynExpand (tUInt newMaxSize) {
    return Resize ((GetMaxSize() + CSTRING_RESIZE_ALLOCATE_OFFSET > newMaxSize) ?
                    GetMaxSize() + CSTRING_RESIZE_ALLOCATE_OFFSET : newMaxSize);
}

/*===========================================================================*\
\*===========================================================================*/

#endif

