/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _DBGINFO_H_
#define _DBGINFO_H_

#include "aTypes.h"
#include "ExtFuncs.h"

/*---------------------------------------------------------------------------*\
    Debug- und Print-Levels
\*---------------------------------------------------------------------------*/

#define Dbg_Level_Nothing           0x00    // Wirklich nichts ausgeben
#define Dbg_Level_Errors            0x01    // Nur Asserts und Errors
#define Dbg_Level_Infos             0x02    // Assert, Errors, Warnungen und Infos
#define Dbg_Level_Short             0x03    // -"- und der Funktionsaufruf (inkl. Parameter)
#define Dbg_Level_Calls             0x04    // -"- und Funktionsaufrufe und -ende (exkl. Parameter)
#define Dbg_Level_Entry             0x05    // -"- und Funktionsaufrufe und -ende (inkl. Parameter)
#define Dbg_Level_Internal          0x06    // Alles wird ausgegeben

                                            // Print-Ausgabe von:
#define Prt_Level_Assert_NoMore     0x0000  // nur die reinen Ausgaben von Assert & PrintErr
#define Prt_Level_Assert_CurFunc    0x0001  // alles aus der aktuellen Funktion
#define Prt_Level_Assert_AllFunc    0x0002  // alles aus allen Funktionen

#define Prt_Level_Warning_NoMore    0x0000  // s.o. fr Warnungen
#define Prt_Level_Warning_CurFunc   0x0004
#define Prt_Level_Warning_AllFunc   0x0008

#define Prt_Level_Info_NoMore       0x0000  // s.o. fr Infos
#define Prt_Level_Info_CurFunc      0x0010
#define Prt_Level_Info_AllFunc      0x0020

#define Prt_Level_NoFunctionName    0x0000  // Keine Ausgabe des Funktonsnamen
#define Prt_Level_NameInEveryLine   0x0040  // Name der Funkton bei jeder
                                            // Print-Ausgabe angeben
#define Prt_Level_NameAtFuncStart   0x0080  // Name der Funktion nur am Funktions-
                                            // Ein- & Austrittspunkt angeben
#define Prt_Level_FileNameAtStart   0x0100  // Dateiname nur beim Funktionsaufruf
#define Prt_Level_FileNameAtInfo    0x0200  // Dateiname vor jeder Debug-Ausgabe
#define Prt_Level_FuncNameAtError   0x0400  // Funktionsname statt Dateiname bei Fehlern


#define Prt_Level_Default           (Prt_Level_NameAtFuncStart |    \
                                     Prt_Level_Assert_AllFunc  |    \
                                     Prt_Level_Warning_CurFunc |    \
                                     Prt_Level_Info_NoMore     |    \
                                     Prt_Level_FileNameAtInfo)

#define Prt_Level_HighDebug         (Prt_Level_NameAtFuncStart |    \
                                     Prt_Level_Assert_AllFunc  |    \
                                     Prt_Level_Warning_AllFunc |    \
                                     Prt_Level_Info_AllFunc    |    \
                                     Prt_Level_FileNameAtInfo)

#define Prt_Level_LowDebug          (Prt_Level_NameInEveryLine |    \
                                     Prt_Level_Assert_NoMore   |    \
                                     Prt_Level_Warning_NoMore  |    \
                                     Prt_Level_Info_NoMore     |    \
                                     Prt_Level_FileNameAtInfo)

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define Prt_Level_Mask_NoMore       0x00    // Nur aktuellen Aufruf ausgegeben
#define Prt_Level_Mask_CurFunc      0x01    // Alle Aufrufe der Func ausgeben
#define Prt_Level_Mask_AllFunc      0x02    // Alle Aufrufe Aller Funcs ausgeben


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define _USE_DEBUG_INFO_CLASS_NONE          0       // Reihenfolge wichtig!!
#define _USE_DEBUG_INFO_CLASS_EMULATION     1       // noch nicht ganz implementiert
#define _USE_DEBUG_INFO_CLASS_MINIMAL       2
#define _USE_DEBUG_INFO_CLASS_NORMAL        3
#define _USE_DEBUG_INFO_CLASS_FULL          4

#ifndef USE_DEBUG_INFO_CLASS
#error USE_DEBUG_INFO_CLASS not defined!
#endif

/*---------------------------------------------------------------------------*\
    dwriteE & dwriteI
\*---------------------------------------------------------------------------*/
#if (USE_DEBUG_INFO_CLASS == _USE_DEBUG_INFO_CLASS_NONE)

#define dwriteE(DL, format)
#define dwriteI(DL, format)

#else

void dwriteE (unsigned DL, char *format, ...);      // Entry
void dwriteI (unsigned DL, char *format, ...);      // Internal

#endif

#if (USE_DEBUG_INFO_CLASS == _USE_DEBUG_INFO_CLASS_NONE)
/*---------------------------------------------------------------------------*\
    Release Makros
\*---------------------------------------------------------------------------*/

#define dhead(name, DL)
#define dheadP(name, DL, PL)
#define dprintBuf(data, length)
#define dassert(expr)                       DbgIAssert (expr)
#define dwarning(expr)                      DbgIAssert (expr)
#define RETURN(format, retval)              return (retval)


#ifdef __linux__

#define dparams(format, x...)
#define dprint(format, x...)
#define dprintWrn(format, x...)
#define dprintErr(format, x...)             DbgIAssert (0)
#define dinfo(expression, format, x...)

#else

#pragma warning (disable: 4002)             // Ignore: "too many parameters for macro"
#define dparams(format)
#define dprint(format)
#define dprintWrn(format)
#define dprintErr(format)                   DbgIAssert (0)
#define dinfo(expression, format)

#endif


#elif (USE_DEBUG_INFO_CLASS == _USE_DEBUG_INFO_CLASS_EMULATION)
/*---------------------------------------------------------------------------*\
    Debug-Ausgaben direkt ohne die DebugInfoClass
    TODO: noch nicht vollstndig implementiert!!!!!
\*---------------------------------------------------------------------------*/

#include "ExtFuncs.h"


#define dhead(name, DL)             char *_CDI_Object_Name = (name);            \
                                    DbgIPrintFull (NAMEATFUNCSTART_BEGIN "%s"   \
                                                  NAMEATFUNCSTART_END,          \
                                                  _CDI_Object_Name);
#define dheadP(name, DL, PL)        char *_CDI_Object_Name = (name);            \
                                    DbgIPrintFull (NAMEATFUNCSTART_BEGIN "%s"   \
                                                  NAMEATFUNCSTART_END,          \
                                                  _CDI_Object_Name);

void dparams (const char *format, ...);

#define dprint                      DbgIPrintFull

void dprintBuf (const void *data, unsigned length);

#define dprintWrn                   DbgIPrintFull

#define dprintErr                   DbgIPrintFull

#define dassert(expr)               DbgIAssert (expr)

#define dwarning(expr)              DbgIAssert (expr)

int dinfo (const int expression, const char *format, ...);

#define RETURN(format, retval)      DbgIPrintFull (RETURNVALUE_BEGIN "%x"       \
                                        RETURNVALUE_END NAMEATFUNCEND_BEGIN     \
                                        "%s" NAMEATFUNCEND_END, (retval),       \
                                        _CDI_Object_Name);                      \
                                    return (retval)


#elif (USE_DEBUG_INFO_CLASS >= _USE_DEBUG_INFO_CLASS_MINIMAL)
/*---------------------------------------------------------------------------*\
    Includes nur frs Debuggen
\*---------------------------------------------------------------------------*/

#include <stdarg.h>
#include "CDbgIDS.h"

/*---------------------------------------------------------------------------*\
    Debug Makros & Funktionen
\*---------------------------------------------------------------------------*/

#define dhead(name, DL)             CDebugInfo _CDI_Object (name, DL, Prt_Level_Default, __FILE__, __LINE__)
#define dheadP(name, DL, PL)        CDebugInfo _CDI_Object (name, DL, PL, __FILE__, __LINE__)


//void dparams (const char *format, ...);
#define dparams                     _CDI_Object.PrintParams


//int dprint (const char *format, ...);
#define dprint                      _CDI_Object.print


#define dprintBuf(data, length)     _CDI_Object.printBuf (data, length)


#define dprintWrn                   _CDI_Object.printWrn


#define dprintErr                   _CDI_Object.printErr


#define dassert(expr)               (void) ((expr) || (_CDI_Object.assertion (#expr, __LINE__), 0))

#define dwarning(expr)              (void) ((expr) || (_CDI_Object.warning (#expr, __LINE__), 0))


//int dinfo (const int expression, const char *format, ...);
#define dinfo                       _CDI_Object.info


#define RETURN(format, retval)      _CDI_Object.SetRetVal ((format), (retval));     \
                                    return (retval)


/*---------------------------------------------------------------------------*\
    CDebugInfo - Class
\*---------------------------------------------------------------------------*/

class CDebugInfo {
public:
    CDebugInfo (char *FuncName, unsigned DebugLevel, unsigned PrintLevel,
                char *_FileName = "?", unsigned _LineNumber = 0);
    virtual ~CDebugInfo (void);

    int print (const char *format, ...);
    void printBuf (const void *data, unsigned length);

    int printWrn (const char *format, ...);
    int printErr (const char *format, ...);

    void assertion (const char *exp, const unsigned int line);
    void assertion (const int expValue, const char *exp, const char *fname, const unsigned int line);

    void warning (const char *output, const unsigned int line);
    void warning (const int expression, const char *output, const char *fname, const unsigned int line);

    int info (const int expression, const char *format, ...);

    void PrintParams (const char *format, ...);
    void SetRetVal (const char format, ...);

    void ShowAllOutputs (void);

protected:
    int RememberOutput (const char *format, ...);

private:
    char *name;
    char *filename;
    unsigned linenumber;
    unsigned debuglevel;
    unsigned printlevel;
    unsigned spaces;

    #if (USE_DEBUG_INFO_CLASS >= _USE_DEBUG_INFO_CLASS_FULL)
    unsigned hCallStack;
    #endif

    #if (USE_DEBUG_INFO_CLASS >= _USE_DEBUG_INFO_CLASS_NORMAL)
    CDbgIDynString Output;
    #endif

    struct RetValueStruct {
        char format;
        union  {
            void *p;
            char *s;
            char  c;
            tUInt u;
            tSInt i;
            tBool b;
        };
    } RetValue;

    void DbgPrintIntern (unsigned PrtLevelMask);
};

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
#if (USE_DEBUG_INFO_CLASS <= _USE_DEBUG_INFO_CLASS_MINIMAL)

inline void CDebugInfo::DbgPrintIntern (unsigned) {
    /*----- Nothing to do! -----*/
}

#endif

#endif /* (USE_DEBUG_INFO_CLASS) */
/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#endif
