/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "DbgStack.h"
#include "DbgInfo.h"
#include "ExtFuncs.h"

#if (USE_DEBUG_INFO_CLASS >= _USE_DEBUG_INFO_CLASS_FULL)

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

CProtectionLock *CDebugStack::pProtect = 0;

CDebugInfo *CDebugStack::array[CDEBUGSTACK_MAXCOUNT] = { 0 };

unsigned CDebugStack::count = 0;
unsigned CDebugStack::root  = 0;
unsigned CDebugStack::top   = 0;


/*===========================================================================*\
\*===========================================================================*/

unsigned CDebugStack::AddTo (CDebugInfo *pDbgInfo) {
    DbgIAssert (pDbgInfo != 0);
    DbgIAssert (top < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (root < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (count <= CDEBUGSTACK_MAXCOUNT);

    if (!pProtect) {
        pProtect = new CProtectionLock (1);
        if (!pProtect || !pProtect->CountLocks()) {
            return 0;
        }
    }
    pProtect->Start();

    unsigned fret = top + 1;
    array[top] = pDbgInfo;

    count++;
    top++;
    top &= CDEBUGSTACK_MAXCOUNT - 1;
    if (top == root) {
        count--;
        root++;
        root &= CDEBUGSTACK_MAXCOUNT - 1;
    }

    pProtect->End();
    return fret;
}


/*===========================================================================*\
\*===========================================================================*/

void CDebugStack::Remove (unsigned hDbgInfo) {
    DbgIAssert (top < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (root < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (count <= CDEBUGSTACK_MAXCOUNT);
    if (hDbgInfo) {
        DbgIAssert (count > 0);
        DbgIAssert (hDbgInfo <= CDEBUGSTACK_MAXCOUNT);
        DbgIAssert (array[hDbgInfo] != 0);
        DbgIAssert (pProtect != 0);
        pProtect->Start();
        array[hDbgInfo - 1] = 0;
        count--;

        if (count) {
            if (top == (hDbgInfo & (CDEBUGSTACK_MAXCOUNT - 1))) {
                hDbgInfo += CDEBUGSTACK_MAXCOUNT - 2;
                hDbgInfo &= CDEBUGSTACK_MAXCOUNT - 1;
                while (array[hDbgInfo] == 0) {
                    DbgIAssert (hDbgInfo != root);
                    hDbgInfo += CDEBUGSTACK_MAXCOUNT - 1;   
                    hDbgInfo &= CDEBUGSTACK_MAXCOUNT - 1;
                }
                top = (hDbgInfo + 1) & (CDEBUGSTACK_MAXCOUNT - 1);
            }
        } else {
            top = root = 0;
        }
        pProtect->End();
    }
}


/*===========================================================================*\
\*===========================================================================*/

void CDebugStack::ShowAllDebugOutputs (void) {
    DbgIAssert (top < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (root < CDEBUGSTACK_MAXCOUNT);
    DbgIAssert (count <= CDEBUGSTACK_MAXCOUNT);
    if (count) {
        DbgIAssert (pProtect != 0);
        pProtect->Start();
        CDebugInfo *dummy;
        unsigned lauf = root;
        while (lauf != top) {
            dummy = array[lauf];
            if (dummy) {
                pProtect->End();
                dummy->ShowAllOutputs();
                pProtect->Start();
            }
            lauf++;
            lauf &= CDEBUGSTACK_MAXCOUNT - 1;
        }
        pProtect->End();
    }
}


/*===========================================================================*\
\*===========================================================================*/

#endif

