/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "PtrList.h"
#include "osImport.h"

/*===========================================================================*\
\*===========================================================================*/

void CPointerListElement::AddAfter (CPointerListElement *pElement) {
    dhead ("CPointerListElement::AddAfter", DEBUG_CPointerList);
    dparams ("%x", pElement);
    dassert (pElement != 0);
    dassert (pElement->pNext == 0);
    dassert (pElement->pPrev == 0);
    pElement->SetProtection (pProtect);
    pElement->pPrev = this;
    pProtect->Begin();
    if (pNext) {
        pNext->pPrev    = pElement;
        pElement->pNext = pNext;
    }
    pNext = pElement;
    pProtect->End();
}

void CPointerListElement::AddBefore (CPointerListElement *pElement) {
    dhead ("CPointerListElement::AddBefore", DEBUG_CPointerList);
    dparams ("%x", pElement);
    dassert (pElement != 0);
    dassert (pElement->pNext == 0);
    dassert (pElement->pPrev == 0);
    pElement->SetProtection (pProtect);
    pElement->pNext = this;
    pProtect->Begin();
    if (pPrev) {
        pPrev->pNext    = pElement;
        pElement->pPrev = pPrev;
    }
    pPrev = pElement;
    pProtect->End();
}

void CPointerListElement::RemoveFromList (void) {
    dhead ("CPointerListElement::RemoveFromList", DEBUG_CPointerList);
    dparams ("%x", this);
    pProtect->Begin();
    if (pPrev) {
        //assert (pPrev->pNext == this);
        pPrev->pNext = pNext;
    }
    if (pNext) {
        //assert (pNext->pPrev == this);
        pNext->pPrev = pPrev;
    }
    pNext = 0;
    pPrev = 0;
    pProtect->End();
}

/*===========================================================================*\
\*===========================================================================*/

tUInt CPointerList::CountElements (void) {
    tUInt count = 0;
    for (CPointerListElement *lauf = Head.GetNext(); lauf != 0; lauf = lauf->GetNext()) {
        count++;
    }
    return count;
}

CPointerListElement *CPointerList::RemoveFirst (void) {
    dhead ("CPointerList::RemoveFirst", DEBUG_CPointerList);
    CPointerListElement *fret = Head.GetNext();
    if (fret) {
        fret->RemoveFromList();
    }
    return fret;
}

CPointerListElement *CPointerList::RemoveLast (void) {
    dhead ("CPointerList::RemoveLast", DEBUG_CPointerList);
    CPointerListElement *fret = Tail.GetPrev();
    if (fret) {
        fret->RemoveFromList();
    }
    return fret;
}

void CPointerList::RemoveAndDeleteAll (void) {
    dhead ("CPointerList::RemoveAndDeleteAll", DEBUG_CPointerList);
    CPointerListElement *pDelItem;
    while ((pDelItem = GetLast()) != 0) {
        delete pDelItem;
    }
}

CPointerListElement *CPointerList::Find (ctFindFunc func, void *RefData) {
    dhead ("CPointerList::Find", DEBUG_CPointerList);
    for (CPointerListElement *pLauf = GetFirst(); (pLauf != 0); pLauf = pLauf->GetNext()) {
        switch (func (pLauf, RefData)) {
        case vYes:
            return pLauf;
        case vCancel:
        case vError:
            return 0;
        default:
            dassert (0);
            // no break!
        case vNo:
            // Nothing to do
            break;
        }
    }
    return 0;
}

/*===========================================================================*\
\*===========================================================================*/
