/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "StdArray.h"
#include "osImport.h"
#include "aString.h"

/*===========================================================================*\
    CStandardArray - Constructor
\*===========================================================================*/

CStandardArray::CStandardArray (tUInt MaxCountOfItems, tUInt SizeOfItem) {
    dhead   ("CStandardArray-Constructor", DEBUG_CStandardArray);
    dassert (MaxCountOfItems > 0);
    dassert (SizeOfItem      > 0);

    // The stuff value ensure that each item will be alinged at pointer boundaries.
    tUInt stuff = (sizeof (void*) - (SizeOfItem % sizeof (void*))) % sizeof (void*);
    sizeCopy    = SizeOfItem;
    sizeItem    = SizeOfItem + stuff;
    dassert ((MaxCountOfItems / 10) * ((sizeItem)/ 10) <= (MAXVAL_tUInt / 100));

    count      = 0;
    emptyCount = 0;
    Array      = new tUChar [MaxCountOfItems * sizeItem];
    ArrayEmpty = new tUChar [MaxCountOfItems           ];
    if (Array && ArrayEmpty) {
        maxCount = MaxCountOfItems;
    } else {
        delete [] Array;
        delete [] ArrayEmpty;
        Array      = 0;
        ArrayEmpty = 0;
        maxCount   = 0;
    }
    dprint ("sizeItem = %x, stuff = %x", sizeItem, stuff);
}


/*===========================================================================*\
    CStandardArray - Destructor
\*===========================================================================*/

CStandardArray::~CStandardArray (void) {
    dhead ("CStandardArray-Destructor", DEBUG_CStandardArray);
    tUChar *tmpArray;
    tUChar *tmpArrayEmpty;
    Protect.Begin();
    sizeItem      = 0;
    count         = 0;
    emptyCount    = 0;
    maxCount      = 0;
    tmpArray      = Array;
    tmpArrayEmpty = ArrayEmpty;
    Array         = 0;
    Protect.End();
    delete [] tmpArray;
    delete [] tmpArrayEmpty;
}


/*===========================================================================*\
    CStandardArray::Set
\*===========================================================================*/

tUInt CStandardArray::Set (tUInt pos, void *dataIn, void *dataOut) {
    dhead ("CStandardArray::Set", DEBUG_CStandardArray);
    dparams ("%x,%p,%p", pos, dataIn, dataOut);
    dassert (dataIn != 0);
    tUInt fret = 0;
    Protect.Begin();
    if (count > pos) {
        fret++;
        if (ArrayEmpty[pos] != 1) {
            if (dataOut) {
                fret++;
                s_memcpy (dataOut, Array + pos * sizeItem, sizeCopy);
            }
        } else {
            dassert (emptyCount > 0);
            ArrayEmpty[pos] = 0;
            emptyCount--;
        }
        s_memcpy (Array + pos * sizeItem, dataIn, sizeCopy);
        Protect.End();
    } else if (count == pos) {
        Protect.End();
        fret = (Append (dataIn) == vFalse) ? 0 : ((!dataOut) ? 1 : 2);
    } else {
        Protect.End();
    }
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::SetEmpty
\*===========================================================================*/

tUInt CStandardArray::SetEmpty (void *dataIn, tUInt *pPos) {
    dhead ("CStandardArray::SetEmpty", DEBUG_CStandardArray);
    dparams ("%p,%p", dataIn, pPos);
    dassert (dataIn != 0);
    Protect.Begin();
    if (emptyCount) {
        tUInt pos;
        for (tUInt i = 0; i < count; i++) {
            pos = i * sizeItem;
            if (ArrayEmpty[i] == 1) {
                emptyCount--;
                ArrayEmpty[i] = 0;
                s_memcpy (Array + pos, dataIn, sizeCopy);
                if (pPos) {
                    *pPos = i;
                }
                Protect.End();
                return 1;
            }
        }
    }
    Protect.End();
    return (Append (dataIn, pPos) == vFalse) ? 0 : 2;
}


/*===========================================================================*\
    CStandardArray::Append
\*===========================================================================*/

tBool CStandardArray::Append (void *dataIn, tUInt *pPos) {
    dhead ("CStandardArray::Append", DEBUG_CStandardArray);
    dparams ("%p,%p", dataIn, pPos);
    dassert (dataIn != 0);
    tBool fret = vFalse;
    Protect.Begin();
    if (count < maxCount) {
        tUInt pos = count * sizeItem;
        ArrayEmpty[count] = 0;
        s_memcpy (Array + pos, dataIn, sizeCopy);
        if (pPos) {
            *pPos = count;
        }
        count++;
        fret = vTrue;
    }
    Protect.End();
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::Ins
\*===========================================================================*/

tBool CStandardArray::Ins (tUInt pos, void *dataIn) {
    dhead ("CStandardArray::Ins", DEBUG_CStandardArray);
    dparams ("%x,%p", pos, dataIn);
    tBool fret = vFalse;
    Protect.Begin();
    if (pos == count) {
        Protect.End();
        fret = Append (dataIn);
    } else {
        if ((pos < count) && (count < maxCount)) {
            tUInt len = count - pos;
            s_strmov (ArrayEmpty + pos, sizeof (tUChar), len);
            ArrayEmpty[pos] = 0;
            pos *= sizeItem;
            s_strmov (Array + pos, sizeItem, len * sizeItem);
            s_memcpy (Array + pos, dataIn, sizeCopy);
            count++;
            fret = vTrue;
        }
        Protect.End();
    }
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::Get
\*===========================================================================*/

tBool CStandardArray::Get (tUInt pos, void *dataOut) {
    dhead ("CStandardArray::Get", DEBUG_CStandardArray);
    dparams ("%x,%p", pos, dataOut);
    dassert (dataOut != 0);
    tBool fret = vFalse;
    tUInt ix   = pos;

    Protect.Begin();
    if (pos < count) {
        pos *= sizeItem;
        if (ArrayEmpty[ix] != 1) {
            s_memcpy (dataOut, Array + pos, sizeCopy);
            fret = vTrue;
        }
    }
    Protect.End();
    return fret;
}


/*===========================================================================*\
    CStandardArray::Get_Direct
\*===========================================================================*/

const void *CStandardArray::Get_Direct (tUInt pos) {
    dhead ("CStandardArray::Get_Direct", DEBUG_CStandardArray);
    dparams ("%x", pos);
    void *fret = 0;
    tUInt ix   = pos;

    Protect.Begin();
    if (pos < count) {
        pos *= sizeItem;
        if (ArrayEmpty[ix] != 1) {
            fret = Array + pos;
        }
    }
    Protect.End();
    return fret;
}


/*===========================================================================*\
    CStandardArray::GetNext
\*===========================================================================*/

tBool CStandardArray::GetNext (void *dataOut, tUInt *FindPos, tUInt StartPos) {
    dhead ("CStandardArray::GetNext", DEBUG_CStandardArray);
    dparams ("%p,%p,%x", dataOut, FindPos, StartPos);
    tBool fret = vFalse;
    tUInt pos  = 0;

    Protect.Begin();
    for (; StartPos < count; StartPos++) {
        pos = StartPos * sizeItem;
        if (ArrayEmpty[StartPos] != 1) {
            if (dataOut) {
                s_memcpy (dataOut, Array + pos, sizeCopy);
            }
            if (FindPos) {
                *FindPos = StartPos;
            }
            fret = vTrue;
            break;
        }
    }
    Protect.End();
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::GetNext_Direct
\*===========================================================================*/

const void *CStandardArray::GetNext_Direct (tUInt *FindPos, tUInt StartPos) {
    dhead ("CStandardArray::GetNext_Direct", DEBUG_CStandardArray);
    dparams ("%p,%x", FindPos, StartPos);
    void *fret = 0;
    tUInt pos  = 0;

    Protect.Begin();
    for (; StartPos < count; StartPos++) {
        pos = StartPos * sizeItem;
        if (ArrayEmpty[StartPos] != 1) {
            if (FindPos) {
                *FindPos = StartPos;
            }
            fret = Array + pos;
            break;
        }
    }
    Protect.End();
    return fret;
}


/*===========================================================================*\
    CStandardArray::Del
\*===========================================================================*/

tBool CStandardArray::Del (tUInt pos, void *dataOut) {
    dhead ("CStandardArray::Del", DEBUG_CStandardArray);
    tBool fret = vFalse;
    Protect.Begin();
    if (pos + 1 < count) {
        if (ArrayEmpty[pos] == 1) {
            dassert (emptyCount > 0);
            emptyCount--;
        } else if (dataOut) {
            s_memcpy (dataOut, Array + pos * sizeItem, sizeCopy);
        }
        s_strmov (ArrayEmpty + pos + 1, -1, count - pos - 1);
        s_strmov (Array + (pos + 1) * sizeItem, -sizeItem, (count - pos - 1) * sizeItem);
        count--;
        fret = vTrue;
        Protect.End();
    } else if (pos + 1 == count) {
        Protect.End();
        fret = DelLast (dataOut);
    } else {
        Protect.End();
    }
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::DelLast
\*===========================================================================*/

tBool CStandardArray::DelLast (void *dataOut) {
    dhead ("CStandardArray::DelLast", DEBUG_CStandardArray);
    tBool fret = vFalse;
    Protect.Begin();
    if (count) {
        count--;
        if (ArrayEmpty[count] == 1) {
            dassert (emptyCount > 0);
            emptyCount--;
        } else if (dataOut) {
            s_memcpy (dataOut, Array + count * sizeItem, sizeCopy);
        }
        fret = vTrue;
    }
    Protect.End();
    return fret;
}


/*===========================================================================*\
    CStandardArray::DelAll
\*===========================================================================*/

void CStandardArray::DelAll (void) {
    dhead ("CStandardArray::DelAll", DEBUG_CStandardArray);
    Protect.Begin();
    count = 0;
    emptyCount = 0;
    Protect.End();
}


/*===========================================================================*\
    CStandardArray::Clear
\*===========================================================================*/

tBool CStandardArray::Clear (tUInt pos, void *dataOut) {
    dhead ("CStandardArray::Clear", DEBUG_CStandardArray);
    tBool fret = vFalse;
    Protect.Begin();
    if (pos + 1 < count) {
        if (ArrayEmpty[pos] != 1) {
            ArrayEmpty[pos] = 1;
            emptyCount++;
            dassert (emptyCount <= count);
            fret = vTrue;
            if (dataOut) {
                s_memcpy (dataOut, Array + pos * sizeItem, sizeCopy);
            }
        }
        Protect.End();
    } else if (pos + 1 == count) {
        Protect.End();
        fret = ClearLast (dataOut);
    } else {
        Protect.End();
    }
    return fret;
}


/*===========================================================================*\
    CStandardArray::ClearLast
\*===========================================================================*/

tBool CStandardArray::ClearLast (void *dataOut) {
    dhead ("CStandardArray::ClearLast", DEBUG_CStandardArray);
    tBool fret = vFalse;
    Protect.Begin();
    if (count) {
        tUInt pos = count - 1;
        if (ArrayEmpty[pos] != 1) {
            ArrayEmpty[pos] = 1;
            emptyCount++;
            dassert (emptyCount <= count);
            fret = vTrue;
            if (dataOut) {
                s_memcpy (dataOut, Array + pos * sizeItem, sizeCopy);
            }
        }
    }
    Protect.End();
    return fret;
}


/*===========================================================================*\
    CStandardArray::ClearAll
\*===========================================================================*/

void CStandardArray::ClearAll (void) {
    dhead ("CStandardArray::ClearAll", DEBUG_CStandardArray);
    Protect.Begin();
    for (tUInt i = 0; i < count; i++) {
        ArrayEmpty[i] = 1;
    }
    emptyCount = count;
    Protect.End();
}


/*===========================================================================*\
    CStandardArray::Find
\*===========================================================================*/

tResult CStandardArray::Find (tArrayCompFunc CompFunc, void *ItemFrag, void *dataOut,
                              tUInt *FindPos, tUInt StartPos) {
    dhead ("CStandardArray::Find", DEBUG_CStandardArray);
    dparams ("%p,%p,%p,%p,%x", CompFunc, ItemFrag, dataOut, FindPos, StartPos);
    dassert (CompFunc != 0);
    dassert (sizeItem > 0);
    dwarning (Array != 0);
    tResult fret = vNo;
    tUChar *lauf = 0;

    Protect.Begin();
    for (lauf = Array + (StartPos * sizeItem); StartPos < count; StartPos++, lauf += sizeItem) {
        if (ArrayEmpty [StartPos] != 1) {
            fret = CompFunc (lauf, ItemFrag);
            if (fret != vNo) {
                if (FindPos) {
                    *FindPos = StartPos;
                }
                if (dataOut) {
                    s_memcpy (dataOut, lauf, sizeCopy);
                }
                break;
            }
        }
    }
    Protect.End();
    RETURN ('x', fret);
}


/*===========================================================================*\
    CStandardArray::Resize
\*===========================================================================*/

tBool CStandardArray::Resize (tUInt newMaxItemCount) {
    dhead ("CStandardArray::Resize", DEBUG_CStandardArray);
    dparams ("%x,old=%x,count=%x", newMaxItemCount, maxCount, count);
    dassert (newMaxItemCount > 0);
    dassert ((newMaxItemCount / 10) * (sizeItem / 10) <= (MAXVAL_tUInt / 100));
    tBool   fret          = vFalse;
    tUChar *newArray      = new tUChar [newMaxItemCount * sizeItem];
    tUChar *newArrayEmpty = new tUChar [newMaxItemCount           ];
    tUChar *delArray      = newArray;
    tUChar *delArrayEmpty = newArrayEmpty;

    Protect.Begin();
    if ((count < newMaxItemCount) && newArray && newArrayEmpty) {
        if (count) {
            s_memcpy (newArray     , Array     , count * sizeItem);
            s_memcpy (newArrayEmpty, ArrayEmpty, count           );
        }
        maxCount      = newMaxItemCount;
        delArray      = Array;
        delArrayEmpty = ArrayEmpty;
        Array         = newArray;
        ArrayEmpty    = newArrayEmpty;
        fret          = vTrue;
    }
    Protect.End();
    delete [] delArray;
    delete [] delArrayEmpty;
    RETURN ('x', fret);
}


/*===========================================================================*\
\*===========================================================================*/
