/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "aStdLib.h"
#include <string.h>

/*---------------------------------------------------------------------------*\
    Interne Variablen
\*---------------------------------------------------------------------------*/

#define PRINTFLAGS_ACTIVE       0x001        // Placeholder found
#define PRINTFLAGS_READY        0x002        // Placeholder fully insert
#define PRINTFLAGS_MINUS        0x004        // Output left oriented
#define PRINTFLAGS_PLUS         0x008        // Use sign '+' or '-'
#define PRINTFLAGS_ZERO         0x010        // Fill with leading zeros
#define PRINTFLAGS_BLANK        0x020        // Use sign '-' or ' '
#define PRINTFLAGS_HASH         0x040        // Show 0 or 0x before numbers
#define PRINTFLAGS_LARGE        0x080        // Placeholder is in LARGE-format
#define PRINTFLAGS_SHORT        0x100        // Placeholder is in SHORT-format
#define PRINTFLAGS_FAR          0x200        // Placeholder is a FAR-pointer
#define PRINTFLAGS_NEAR         0x400        // Placeholder is a NEAR-pointer

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static const char DigitForStrings[16] =  {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'A', 'B', 'C', 'D', 'E', 'F'
};


/*---------------------------------------------------------------------------*\
    internal functions for "vsnprintf" !
\*---------------------------------------------------------------------------*/

static void ptoa (void *pointer, char *string);

static unsigned ltoanX (tSLong value, char *string, tUInt maxlen, tSInt radix,
                        tUInt width, tUInt flags);


/*---------------------------------------------------------------------------*\
    Convert:  String -> Long
\*---------------------------------------------------------------------------*/

long atol (char *string) {
    long value = 0;
    long sign = 1;

    //ASSERT (string != 0);

    while (*string == ' ') {
        string++;
    }
    switch (*string) {
    case '-':
        sign = -1;
        // kein break!!
    case '+':
        string++;
        break;
    }
    for (; ((*string != '\0') && (*string >= '0') && (*string <= '9')); string++) {
        value = value * 10 + *string - '0';
    }
    return value * sign;
}


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

long a_strtoln (const char *nptr, const char **endptr, unsigned maxlength, int base) {
    long value = 0;
    long sign = 1;
    unsigned char figure;

    //ASSERT (nptr != 0);
    //ASSERT ((base >= 2) && (base <= 16));
    //ASSERT (maxlength != 0);

    while ((*nptr == ' ')) {
        nptr++;
    }
    switch (*nptr) {
    case '-':
        sign = -1;
        // no break!!
    case '+':
        nptr++;
        break;
    }
    for (; ((*nptr != '\0') && (maxlength)); nptr++, maxlength--) {
        figure = (unsigned char) (*nptr - '0');
        if (figure >= ('A'-'0')) {
            figure = (unsigned char)(figure - ((figure >= 'a'-'0') ? 'a'-'0'-10 : 'A'-'0'-10));
        }
        if (figure >= base) {
            break;
        }
        value = value * base + figure;
    }
    if (endptr) {
        *endptr = nptr;
    }
    return value * sign;
}


/*---------------------------------------------------------------------------*\
    Convert: Long -> String
\*---------------------------------------------------------------------------*/

unsigned ltoan (tSInt value, char *string, tUInt maxlen, tSInt radix) {
    tUInt      uvalue;
    char       buffer[sizeof (tPLong) * 8];
    unsigned   bufpos = sizeof (buffer) - 1;
    char       prechar = '\0';

    // ASSERT (string != 0);
    // ASSERT ((radix <= 16) && (radix >= -16) && (radix != 0) && (radix != 1) && (radix != -1));
    // ASSERT (maxlen > 1);

    if (radix < 0) {
        radix *= -1;
        if (value < 0) {
            value *= -1;
            prechar = '-';
            maxlen--;
        }
    }
    uvalue = (tUInt) value;

    while (uvalue >= (tUInt)radix) {
        buffer[bufpos--] = DigitForStrings[uvalue % radix];
        uvalue /= radix;
    }
    buffer[bufpos] = DigitForStrings[uvalue % radix];

    bufpos = sizeof (buffer) - bufpos;

    if (maxlen <= bufpos) {
        // Platz fr String zu klein
        *string = '\0';
        return 0;
    }

    // Konvertierung in den String kopieren
    if (prechar) {
        *string++ = prechar;
    }
    memcpy (string, buffer + sizeof (buffer) - bufpos, bufpos);

    string[bufpos] = '\0';
    return bufpos + (prechar) ? 1 : 0;
}


/*---------------------------------------------------------------------------*\
    String Conversion
\*---------------------------------------------------------------------------*/

char *aStrUpr (char *pString) {
    if (pString) {
        char *lauf;
        for (lauf = pString; *lauf != '\0'; lauf++) {
            if ((*lauf >= 'a') && (*lauf <= 'z')) {
                *lauf -= ('a' - 'A');
            }
        }
    }
    return pString;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static int t_vsnprintf (char *buffer, const unsigned maxcount, const char *format, va_list argptr,
                        unsigned tabsize, char tabchar) {
    unsigned PrtFlags   = 0;            // Possible Flags for "placeholder"
    unsigned PrtWidth   = 0;            // Width of "placeholder"
    int      PrtPreci   = -2;           // Precision of "placeholder"
    int      PrtValue   = 0;            // Returnvalue of convert function
    const char *savepos = format;
    char       *argchar = 0;
    unsigned    count   = maxcount;     // free place in buffer
    unsigned ErrorFound = 0;

    while ((count > 1) && (*format != 0) && (!ErrorFound)) {
        if (PrtFlags) {

            /*----- exams placeholders (%...) -----*/
            switch (*format) {

            /*----- FLAGS -----*/
            case '-':
                PrtFlags |= PRINTFLAGS_MINUS;
                break;
            case '+':
                PrtFlags |= PRINTFLAGS_PLUS;
                break;
            case ' ':
                PrtFlags |= PRINTFLAGS_BLANK;
                break;
            case '#':
                PrtFlags |= PRINTFLAGS_HASH;
                break;
            case 'l':
            case 'L':
                PrtFlags |= PRINTFLAGS_LARGE;
                break;

            case 'h':
                PrtFlags |= PRINTFLAGS_SHORT;
                break;

            /*----- WIDTH or PRECISION-----*/
            case '0':
                if (PrtPreci == -2) {
                    PrtFlags |= PRINTFLAGS_ZERO;
                }
                // kein break !
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                if (PrtPreci == -2) {
                    PrtWidth = a_strtol (format, &format, 10);
                } else {
                    PrtPreci = a_strtol (format, &format, 10);
                }
                format--;
                break;
            case '*':
                if (PrtPreci == -2) {
                    PrtWidth = va_arg (argptr, unsigned);
                } else {
                    PrtPreci = va_arg (argptr, unsigned);
                }
                break;

            /*----- TYPE -----*/
            case 'a':
                argchar = va_arg (argptr, char *);
                PrtValue = t_vsnprintf (buffer, count, argchar, va_arg (argptr, va_list), tabsize, tabchar);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'b':
                if (va_arg (argptr, char)) {
                    argchar = "TRUE";
                    PrtValue = 4;
                } else {
                    argchar = "FALSE";
                    PrtValue = 5;
                }
                if (PrtPreci == 1) {
                    PrtValue = 1;
                }
                if ((PrtWidth > (unsigned)PrtValue) && !(PrtFlags & PRINTFLAGS_MINUS)) {
                    PrtWidth = (PrtWidth - PrtValue >= count) ? count - 1: PrtWidth - PrtValue;
                    memset (buffer, ' ', PrtWidth);
                    count -= PrtWidth;
                    buffer += PrtWidth;
                }
                if (count > (unsigned)PrtValue) {
                    memcpy (buffer, argchar, PrtValue);
                } else if (count > 1) {
                    /*----- show only 'T'- or 'F' -----*/
                    *buffer = *argchar;
                    PrtValue = 1;
                } else {
                    PrtValue = 0;
                }
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 't':
                tabsize = PrtWidth;
                tabchar = va_arg (argptr, char);

                PrtWidth = 0;
                PrtPreci = -2;
                buffer--;
                count++;
                PrtValue = 1;
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'd':
            case 'i':
                PrtValue = ltoanX ((PrtFlags & PRINTFLAGS_LARGE) ? va_arg (argptr, tSLong) : va_arg (argptr, tSInt),
                                   buffer, count, -10, PrtWidth, PrtFlags);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'u':
                PrtValue = ltoanX ((PrtFlags & PRINTFLAGS_LARGE) ? va_arg (argptr, tULong) : va_arg (argptr, tUInt),
                                   buffer, count, 10, PrtWidth, PrtFlags);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'o':
                PrtValue = ltoanX ((PrtFlags & PRINTFLAGS_LARGE) ? va_arg (argptr, tULong) : va_arg (argptr, tUInt),
                                   buffer, count, 8, PrtWidth, PrtFlags);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'x':
                PrtValue = ltoanX ((PrtFlags & PRINTFLAGS_LARGE) ? va_arg (argptr, tULong) : va_arg (argptr, tUInt),
                                   buffer, count, 0x10, PrtWidth, PrtFlags);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'X':
                PrtValue = ltoanX ((PrtFlags & PRINTFLAGS_LARGE) ? va_arg (argptr, tULong) : va_arg (argptr, tUInt),
                                   buffer, count, 0x10, PrtWidth, PrtFlags);
                if (PrtValue) {
                    aStrUpr (buffer);
                }
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'c':
                if ((PrtWidth > 1) && !(PrtFlags & PRINTFLAGS_MINUS)) {
                    PrtValue = (PrtWidth > count) ? (count - 1) : (PrtWidth - 1);
                    memset (buffer, ' ', PrtValue);
                } else {
                    PrtValue = 0;
                }
                if (count - PrtValue > 1) {
                    buffer[PrtValue] = va_arg (argptr, char);
                    PrtValue++;
                }
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 's':
                argchar = va_arg (argptr, char *);
                if (!argchar) {
                    PrtFlags &= ~PRINTFLAGS_LARGE;  // LARGE-FLAG lschen
                    argchar = "[NULL]";
                }
                if (PrtFlags & PRINTFLAGS_LARGE) {
                    PrtValue = 0;
                    while (argchar[PrtValue * 2] != '\0') {
                        PrtValue++;
                    }
                } else {
                    PrtValue = strlen (argchar);
                }
                if ((PrtWidth > (unsigned) PrtValue) && !(PrtFlags & PRINTFLAGS_MINUS)) {
                    PrtWidth = (PrtWidth - PrtValue >= count) ? (count - 1) : (PrtWidth - PrtValue);
                    memset (buffer, ' ', PrtWidth);
                    count -= PrtWidth;
                    buffer += PrtWidth;
                }
                if ((unsigned) PrtValue >= count) {
                    PrtValue = count - 1;
                }
                if (PrtValue) {
                    if (PrtFlags & PRINTFLAGS_LARGE) {
                        for (ErrorFound = 0; PrtValue - ErrorFound > 0; ErrorFound++) {
                            buffer[ErrorFound] = argchar[ErrorFound * 2];
                        }
                        ErrorFound = 0;
                    }
                    else {
                        memcpy (buffer, argchar, PrtValue);
                    }
                } else {
                    buffer--;
                    count++;
                    PrtValue = 1;
                }
                PrtFlags |= PRINTFLAGS_READY;
                break;
            case 'p':
                if ((PrtWidth > 9) && !(PrtFlags & PRINTFLAGS_MINUS)) {
                    PrtWidth = (PrtWidth - 9 >= count) ? count - 1: PrtWidth - 9;
                    memset (buffer, ' ', PrtWidth);
                    count -= PrtWidth;
                    buffer += PrtWidth;
                }
                PrtValue = ((sizeof (void *) > 4) ? 19 : 9 );
                if (count > (unsigned)PrtValue) {
                    ptoa (va_arg (argptr, void *), buffer);
                } else {
                    PrtValue = 0;
                }
                PrtFlags |= PRINTFLAGS_READY;
                break;

            /*----- Found PRECISION -----*/
            case '.':
                if (PrtPreci == -2) {
                    PrtPreci = -1;
                    break;
                }
                /*----- no Break! -----*/

            /*----- OTHERS: wrong placeholder -----*/
            default:
                PrtValue = (int) (format - savepos + 1);
                if ((unsigned) PrtValue >= count) {
                    PrtValue = count - 1;
                }
                memcpy (buffer, savepos, PrtValue);
                PrtFlags |= PRINTFLAGS_READY;
                break;
            } //end: switch (*format)

            if (PrtFlags & PRINTFLAGS_READY) {
                // placeholder found
                if (PrtValue > 0) {
                    if ((PrtPreci >= 0) && (PrtPreci < PrtValue)) {
                        /*----- TODO: wrong!!! -----*/
                        PrtValue = PrtPreci;
                    }
                    count -= PrtValue;
                    buffer += PrtValue;
                    if ((PrtWidth > (unsigned)PrtValue) && (PrtFlags & PRINTFLAGS_MINUS)) {
                        PrtWidth -= PrtValue;
                        if (PrtWidth >= count) {
                            PrtWidth = count - 1;
                        }
                        count -= PrtWidth;
                        for (; (PrtWidth > 0); PrtWidth--) {
                            *buffer++ = ' ';
                        }
                    }
                } else {
                    ErrorFound = 1;
                }
                PrtFlags = 0;
                PrtWidth = 0;
                PrtPreci = -2;
                PrtValue = 0;
            }

        } else if (*format == '%') {
            //ASSERT (PrtFlags == 0);
            savepos = format;
            if (*(format + 1) == '%') {
                count--;
                *buffer++ = *format++;
            } else {
                PrtValue = 0;
                PrtWidth = 0;
                PrtPreci = -2;
                PrtFlags = PRINTFLAGS_ACTIVE;
            }

        } else {
            count--;
            *buffer++ = *format;
            if ((*format == '\n') && (tabsize > 0) && (tabsize < count) && (tabchar > 0)) {
                memset (buffer, tabchar, tabsize);
                buffer += tabsize;
                count  -= tabsize;
            }
        }
        format++;
    } //end: while

    *buffer = '\0';
    if (ErrorFound) {
        return -1;
    }
    return (maxcount - count);
}


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

int a_vsnprintf (char *buffer, const unsigned maxcount, const char *format, va_list argptr) {
    return t_vsnprintf (buffer, maxcount, format, argptr, 0, 0);
}


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

int a_snprintf (char *buffer, const unsigned maxcount, const char *format, ...) {
    int fret;
    va_list arg_ptr;
    va_start (arg_ptr, format);
    fret = t_vsnprintf (buffer, maxcount, format, arg_ptr, 0, 0);
    va_end (arg_ptr);
    return fret;
}


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static void ptoa (void *pointer, char *string) {
    tPLong p   = (tPLong) pointer;
    char  *fmt = sizeof (p) > 4 ? "    .    :    .    " : "    :    "; // must be symmetric!
    unsigned i = strlen (fmt);

    string[i] = '\0';
    string--;
    for (; i > 0; i--, fmt++) {
        if (*fmt == ' ') {
            string [i] = DigitForStrings[p % 0x10];
            p /= 0x10;
        } else  {
            string [i] = *fmt;
        }
    }
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

unsigned ltoanX (tSLong value, char *string, tUInt maxlen, tSInt radix,
                 tUInt width, tUInt flags) {
    tULong        uvalue;
    char          prechar[3];
    unsigned      precharlen = 0;
    char          buffer[sizeof (tSLong) * 8];
    unsigned      bufpos = sizeof (buffer) - 1;

    // ASSERT (string != 0);
    // ASSERT ((radix <= 16) && (radix >= -16) && (radix != 0) && (radix != 1) && (radix != -1));
    // ASSERT (maxlen > 1);

    if (value < 0) {
        if (radix < 0) {
            value *= -1;
            prechar[precharlen++] = '-';
        }
    } else {
        if (flags & PRINTFLAGS_PLUS) {
            prechar[precharlen++] = '+';
        } else if (flags & PRINTFLAGS_BLANK) {
            prechar[precharlen++] = ' ';
        }
    }
    if (radix < 0) {
        radix *= -1;
    }
    if (flags & PRINTFLAGS_HASH) {
        switch (radix) {
        case 8:
            prechar[precharlen++] = '0';
            break;
        case 16:
            prechar[precharlen++] = '0';
            prechar[precharlen++] = 'x';
            break;
        }
    }

    uvalue = (tULong) value;
    while (uvalue >= (tULong) radix) {
        buffer[bufpos--] = DigitForStrings[uvalue % radix];
        uvalue /= radix;
    }
    buffer[bufpos] = DigitForStrings[uvalue % radix];

    uvalue = bufpos;
    bufpos = sizeof (buffer) - bufpos + precharlen;

    if (maxlen <= bufpos) {
        *string = '0';
        return 0;
    }
    if ((width > bufpos) && !(flags & PRINTFLAGS_MINUS)) {
        width -= bufpos;
        memset (string, ((flags & PRINTFLAGS_ZERO) ? '0' : ' '), width);
        string += width;
    } else {
        width = 0;
    }

    if (precharlen) {
        memcpy (string, prechar, precharlen);
    }
    memcpy (string + precharlen, buffer + uvalue, bufpos - precharlen);

    string[bufpos] = '\0';
    return bufpos + width;
}


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define GetNextValidNumber(tel, min)                                                \
    while (*tel && (tel >= min) &&                                                  \
           ((*tel==' ') || (*tel=='(') || (*tel==')') || (*tel=='+'))) {            \
        tel--;                                                                      \
    }

/*---------------------------------------------------------------------------*/

unsigned StrCmpTN (const char *tel1, const char *tel2) {
    const char *lauf1;
    const char *lauf2;

    if ((tel1 == 0) || (tel2 == 0) || (*tel1 == '\0') || (*tel2 == '\0')) {
        return 0;  // sind identisch !!
    }
    //ASSERT (strlen (tel1) > 0);
    //ASSERT (strlen (tel2) > 0);

    // Setze Zeiger ans Ende des Strings!!
    lauf1 = tel1 + strlen (tel1) - 1;
    lauf2 = tel2 + strlen (tel2) - 1;

    for (;; lauf1--, lauf2--) {
        GetNextValidNumber (lauf1, tel1);
        GetNextValidNumber (lauf2, tel2);
        if ((lauf1 < tel1) || (lauf2 < tel2)) {
            return 0;   // sind identisch !!
        }
        if (*lauf1 != *lauf2) {
            return 1;   // nicht gleich !!
        }
    }
}


#undef GetNextValidNumber
/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
