/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _ASTRING_H_
#define _ASTRING_H_

#include <string.h>
#include "aTypes.h"

/*===========================================================================*\
    String length and size
\*===========================================================================*/

tBool s_strempty (tUChar   *string);
tBool s_strempty (tUWiChar *string);

tUInt s_strlen (tUChar   *string);
tUInt s_strlen (tUWiChar *string);

tUInt s_strsize (tUChar   *string);
tUInt s_strsize (tUWiChar *string);

/*===========================================================================*\
    Copy
\*===========================================================================*/

void s_strncpy (tUChar   *dest, tUChar   *src, tUInt len);
void s_strncpy (tUChar   *dest, tUWiChar *src, tUInt len);
void s_strncpy (tUWiChar *dest, tUChar   *src, tUInt len);
void s_strncpy (tUWiChar *dest, tUWiChar *src, tUInt len);

void s_memcpy (void *dest, void *src, tUInt len);


/*===========================================================================*\
    Move
\*===========================================================================*/

void s_strmov (tUChar   *string, tSInt offset, tUInt len);
void s_strmov (tUWiChar *string, tSInt offset, tUInt len);

void s_memmov (void *dest, tSInt offset, tUInt len);


/*===========================================================================*\
    Find
\*===========================================================================*/

tUChar *s_findfirstchar (tUChar *string, tUChar chSearch);

/*===========================================================================*\
    inline Funktionen
\*===========================================================================*/

inline tBool s_strempty (tUChar *string) {
    return (*string != 0) ? vFalse : vTrue;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tBool s_strempty (tUWiChar *string) {
    return (*string != 0) ? vFalse : vTrue;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt s_strlen (tUChar *string) {
    return strlen ((char *)string);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt s_strlen (tUWiChar *string) {
    tUInt len = 0;
    while (*string != 0) {
        string++;
        len++;
    }
    return len;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt s_strsize (tUChar *string) {
    return (string) ? (s_strlen (string) + 1) : 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt getStrSize (tUWiChar *string) {
    return (string) ? (s_strlen (string) + 1) * sizeof (tUWiChar) : 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_strncpy (tUChar *dest, tUChar *src, tUInt len) {
    memcpy (dest, src, len);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_strncpy (tUChar *dest, tUWiChar *src, tUInt len) {
    for (; len > 0; len--) {
        *dest++ = (tUChar)(*src++);
    }
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_strncpy (tUWiChar *dest, tUChar *src, tUInt len) {
    for (; len > 0; len--) {
        *dest++ = *src++;
    }
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_strncpy (tUWiChar *dest, tUWiChar *src, tUInt len) {
    memcpy (dest, src, len * sizeof (tUWiChar));
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_memcpy (void *dest, void *src, tUInt len) {
    memcpy (dest, src, len);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void s_memmov (void *dest, tSInt offset, tUInt len) {
    s_strmov ((tUChar *)dest, offset, len);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUChar *s_findfirstchar (tUChar *string, tUChar chSearch) {
    return (tUChar *)strchr ((char *)string, chSearch);
}

/*===========================================================================*\
\*===========================================================================*/

#endif
