/*---------------------------------------------------------------------------*\
 * Copyright (C) 2000 AVM GmbH. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT 
 * ANY LIABILITY FOR ANY DAMAGES arising out of or in connection 
 * with the use or performance of this software. See the
 * GNU General Public License for further details.
\*---------------------------------------------------------------------------*/

#ifndef _ATOMAR_H_
#define _ATOMAR_H_

/*===========================================================================*\
\*===========================================================================*/

#ifndef __cplusplus
#error Can only be used under C++ at the moment!
#endif

#include "aTypes.h"

/*===========================================================================*\
    Definition
\*===========================================================================*/

tUInt atomicGet (tUInt *pValue);
tSInt atomicGet (tSInt *pValue);
void *atomicGet (void **ppValue);
tUInt atomicSet (tUInt *pUInt, tUInt value);
tSInt atomicSet (tSInt *pSInt, tSInt value);
void *atomicSet (void **pVoid, void *value);
tUInt atomicAdd (tUInt *pUInt, tUInt value);
tUInt atomicAdd (tUInt *pUInt, tSInt value);
tSInt atomicAdd (tSInt *pSInt, tSInt value);
tUInt atomicSub (tUInt *pUInt, tUInt value);
tUInt atomicExchange (tUInt *pUInt,  tUInt value);
tSInt atomicExchange (tSInt *pSInt,  tSInt value);
void *atomicExchange (void **ppVoid, void *value);

/*===========================================================================*\
    inline implementation:
\*===========================================================================*/

inline tUInt atomicGet (tUInt *pValue) {
    return *pValue;
}

inline tSInt atomicGet (tSInt *pValue) {
    return *pValue;
}

inline void *atomicGet (void **ppValue) {
    return *ppValue;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt atomicSet (tUInt *pUInt, tUInt value) {
    return *pUInt = value;
}

inline tSInt atomicSet (tSInt *pSInt, tSInt value) {
    return *pSInt = value;
}

inline void *atomicSet (void **ppVoid, void *value) {
    return *ppVoid = value;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt atomicAdd (tUInt *pUInt, tUInt value) {
    tUInt old = *pUInt;
    *pUInt += value;
    return old;
}

inline tUInt atomicAdd (tUInt *pUInt, tSInt value) {
    tUInt old = *pUInt;
    *pUInt += value;
    return old;
}

inline tSInt atomicAdd (tSInt *pSInt, tSInt value) {
    tSInt old = *pSInt;
    *pSInt += value;
    return old;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt atomicSub (tUInt *pUInt, tUInt value) {
    tUInt old = *pUInt;
    *pUInt -= value;
    return old;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt atomicExchange (tUInt *pUInt,  tUInt value) {
    tUInt old = *pUInt;
    *pUInt = value;
    return old;
}

inline tSInt atomicExchange (tSInt *pSInt,  tSInt value) {
    tSInt old = *pSInt;
    *pSInt = value;
    return old;
}

inline void *atomicExchange (void **ppVoid, void *value) {
    void *old = *ppVoid;
    *ppVoid = value;
    return old;
}

/*===========================================================================*\
\*===========================================================================*/
#endif

